<?php

namespace App\Http\Controllers;

use App\AdminOtherAccount;
use App\AdminPurchaesInvoice;
use App\AdminTermsAndCondition;
use App\AdminWarehouse;
use App\Company;
use App\Customer;
use App\Expense;
use App\ItemName;
use App\OtherAccountBuyer;
use App\PurchaseInvoiceExpense;
use App\PurchaseOrder;
use App\Quotation;
use App\StaffAndSalesman;
use App\Supplier;
use App\WareHouse;
use App\Branches;
use Carbon\Carbon;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\DB;
use App\Permission;
use App\AdminPeriodClosing;
class AdminPurchaesInvoiceController extends Controller
{
    /**
     * Display a listing of the resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function index()
    {
            $branches = Branches::all();

            $purchaseInvoices = AdminPurchaesInvoice::with('account', 'item', 'expenses', 'warehouse')
            ->whereIn('id', function ($query) {
                $query->selectRaw('MIN(id)')
                    ->from('admin_purchaes_invoices')
                    ->where('branch_id', session()->get('selected_branch'))
                    // ->whereNotNull('invoice_no');
                    ->groupBy('invoice_no');
            })->withTrashed()
            ->where('branch_id', session()->get('selected_branch'))
            ->orderBy('invoice_no', 'desc')->get();


        return view('crusher.trans_purchase_invoice.index', compact('purchaseInvoices','branches'));
    }


    /**
     * Show the form for creating a new resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function create()
    {

         // Retrieve permission titles from Permission table
         $permissionTitles = Permission::where('branch_id', session()->get('selected_branch'))->pluck('name')->toArray();

         // Filter permission titles using hasBranchPermission function
         $filteredPermissionTitles = [];
         $branchId = session()->get('selected_branch');
         foreach ($permissionTitles as $permissionTitle) {
             if (Branches::hasBranchPermission($branchId, $permissionTitle)) {
                 $filteredPermissionTitles[] = $permissionTitle;
             }
         }

        $customers = Customer::where('branch_id', session()->get('selected_branch'))->get();
        $items = ItemName::where('branch_id', session()->get('selected_branch'))->get();
        $warehouses = AdminWarehouse::where('branch_id', session()->get('selected_branch'))->get();
        $expenses = Expense::where(['branch_id'=>session()->get('selected_branch'), 'class_of_account_id'=>'Expenses'])
        ->get()
        ->map(function ($item) {
            return $item->setAttribute('table_name', 'App\Expense');
        });
        $otherAccountExpenses = AdminOtherAccount::where(['branch_id'=>session()->get('selected_branch'), 'class_of_account_id'=>'Expenses'])
        ->get()
        ->map(function ($item) {
            return $item->setAttribute('table_name', 'App\AdminOtherAccount');
        });

        $expensesAccount = $expenses->concat($otherAccountExpenses);

        $currentDate = Carbon::now()->format('Ymd');
        // Get the latest quotation number for the branch
        $latestPi = AdminPurchaesInvoice::withTrashed()->where('branch_id', session()->get('selected_branch'))
            ->latest('invoice_no')->first();

        // Calculate the new quotation number
        if ($latestPi) {
            $lastNumber = (int)substr($latestPi->invoice_no, -4); // Extract the last 4 digits
            $newNumber = $lastNumber + 1;
            $newPiNo = str_pad($newNumber, 4, '0', STR_PAD_LEFT);
        } else {
            $newPiNo = '0001';
        }



        $poNo = request()->input('po_no');

        if ($poNo) {
            $purchaseOrders = PurchaseOrder::where('invoice_no',$poNo)->get();
        } else {
            $purchaseOrders = [];
        }

        // // Retrieve customers and include both 'account_name' and 'table_name' attributes
        // $customers = Customer::where('branch_id', session()->get('selected_branch'))
        // ->whereIn('name_of_organisation', $filteredPermissionTitles)
        // ->select('*', 'name_of_organisation as account_name')
        // ->get()
        // ->map(function ($item) {
        //     return $item->setAttribute('table_name', 'App\Customer');
        // });

        // // Retrieve suppliers and include both 'account_name' and 'table_name' attributes
        // $suppliers = Supplier::where('branch_id', session()->get('selected_branch'))
        // ->whereIn('name_of_organisation', $filteredPermissionTitles)
        // ->select('*', 'name_of_organisation as account_name')
        // ->get()
        // ->map(function ($item) {
        //     return $item->setAttribute('table_name', 'App\Supplier');
        // });

        // // Retrieve expenses and set 'table_name' to the fully-qualified model class name
        // $salesmens = StaffAndSalesman::where('branch_id', session()->get('selected_branch'))
        // ->whereIn(DB::raw("CONCAT(first_name, ' ', last_name)"), $filteredPermissionTitles)
        // ->select('*', DB::raw("CONCAT(first_name, ' ', last_name) as account_name"))
        // ->get()
        // ->map(function ($item) {
        //     return $item->setAttribute('table_name', 'App\StaffAndSalesman');
        // });

        // // Retrieve expenses and set 'table_name' to the fully-qualified model class name
        // $expenses = Expense::where('branch_id', session()->get('selected_branch'))
        // ->whereIn('account_name', $filteredPermissionTitles)
        // ->get()
        // ->map(function ($item) {
        //     return $item->setAttribute('table_name', 'App\Expense');
        // });

        // // Retrieve other accounts and set 'table_name' to the fully-qualified model class name
        // $otherAccounts = AdminOtherAccount::where('branch_id', session()->get('selected_branch'))
        // ->whereIn('account_name', $filteredPermissionTitles)
        // ->get()
        // ->map(function ($item) {
        //     return $item->setAttribute('table_name', 'App\AdminOtherAccount');
        // });

        $customers = Customer::where('branch_id', session()->get('selected_branch'))
        ->whereIn(DB::raw("REPLACE(name_of_organisation, ' ', '-')"), $filteredPermissionTitles)
        ->select('*', DB::raw("(name_of_organisation) as account_name"))
        ->get()
        ->map(function ($item) {
            return $item->setAttribute('table_name', 'App\Customer');
        });

        $suppliers = Supplier::where('branch_id', session()->get('selected_branch'))
        ->whereIn(DB::raw("REPLACE(name_of_organisation, ' ', '-')"), $filteredPermissionTitles)
        ->select('*', DB::raw("(name_of_organisation) as account_name"))
        ->get()
        ->map(function ($item) {
            return $item->setAttribute('table_name', 'App\Supplier');
        });

        $salesmens = StaffAndSalesman::where('branch_id', session()->get('selected_branch'))
        ->whereIn(DB::raw("CONCAT(REPLACE(first_name, ' ', '-'), '-', REPLACE(last_name, ' ', '-'))"), $filteredPermissionTitles)
        ->select('*', DB::raw("CONCAT(first_name, ' ', last_name) AS account_name"))
        ->get()
        ->map(function ($item) {
            return $item->setAttribute('table_name', 'App\StaffAndSalesman');
        });

        $expenses = Expense::where('branch_id', session()->get('selected_branch'))
        ->whereIn(DB::raw("REPLACE(account_name, ' ', '-')"), $filteredPermissionTitles)
        ->select('*', DB::raw("(account_name)"))
        ->get()
        ->map(function ($item) {
            return $item->setAttribute('table_name', 'App\Expense');
        });

        $otherAccounts = AdminOtherAccount::where('branch_id', session()->get('selected_branch'))
        ->whereIn(DB::raw("REPLACE(account_name, ' ', '-')"), $filteredPermissionTitles)
        ->select('*', DB::raw("(account_name)"))
        ->get()
        ->map(function ($item) {
            $item->setAttribute('table_name', 'App\AdminOtherAccount');
            $item->setAttribute('class_of_account', ($item->class_of_account_id == 1 || $item->class_of_account_id == 3) ? '1,3' : '');
            return $item;
        });

        // AdminOtherAccount table ke colmun class_of_account_id me 1 3 value store hai,  setAttribute table_name me set kar do. App\AdminOtherAccount ye hata do

        // Concatenate all collections into a single collection
        $accounts = $customers->concat($expenses)->concat($suppliers)->concat($otherAccounts)->concat($salesmens);

        $periodClosings = AdminPeriodClosing::where('branch_id', session()->get('selected_branch'))->pluck('to')->toArray();


        return view('crusher.trans_purchase_invoice.create', compact('newPiNo', 'customers', 'items', 'warehouses', 'expensesAccount', 'purchaseOrders', 'accounts','periodClosings'));
    }

    /**
     * Store a newly created resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @return \Illuminate\Http\Response
     */
    public function store(Request $request)
    {
        // dd($request);
        $latestNo = AdminPurchaesInvoice::withTrashed()->where('branch_id', session()->get('selected_branch'))
        ->latest('invoice_no')->first();

        if ($latestNo) {
        $latestInvoiceNo = $latestNo->invoice_no;

        $latestNumber = (int)preg_replace('/[^0-9]/', '', $latestInvoiceNo);

        $newNumber = $latestNumber + 1;

        $newInvoiceNo = str_pad($newNumber, 4, '0', STR_PAD_LEFT);
        } else {
        $newInvoiceNo = '0001';
        }
        try {
            $validatedData = $request->validate([
                // 'invoice_no' => 'nullable|string',
                'po_no' => 'nullable|string',
                'lpo_no' => 'nullable|string',
                'lpo_days' => 'nullable|string',
                'term_of_payment' => 'nullable|string',
                'narrations' => 'nullable|string',
                'date' => 'nullable|date',
                'account_id' => 'nullable',
                'acc_no' => 'nullable',
                'account_id' => 'nullable|integer',
                'account_type' => 'nullable',
                'name_account' => 'nullable',
                'item_description' => 'nullable|array',
                'item_description.*' => 'nullable|string',
                'item_id' => 'nullable|array',
                'item_id.*' => 'nullable|exists:item_names,id',
                'item_unit' => 'nullable|array',
                'item_unit.*' => 'nullable|string',
                'warehouse_id' => 'nullable|array',
                'warehouse_id.*' => 'nullable|exists:admin_warehouses,id',
                'quantity' => 'nullable|array',
                'quantity.*' => 'nullable|numeric',
                'rate_unit' => 'nullable|array',
                'rate_unit.*' => 'nullable|numeric',
                'gross_amount' => 'nullable|array',
                'gross_amount.*' => 'nullable|numeric',
                'vat_percentage' => 'nullable|array',
                'vat_percentage.*' => 'nullable|numeric',
                'vat_amount' => 'nullable|array',
                'vat_amount.*' => 'nullable|numeric',
                'net_amount' => 'nullable|array',
                'net_amount.*' => 'nullable|numeric',
                'c_f' => 'nullable|array',
                'c_f.*' => 'nullable|numeric',
                'expense_type' => 'nullable|array',
                'expense_type.*' => 'nullable',
                'account_name' => 'nullable|array',
                'account_name.*' => 'nullable',
                'expense_id' => 'nullable|array',
                'expense_id.*' => 'nullable|integer',
                'expense_amount' => 'nullable|array',
                'expense_amount.*' => 'nullable|numeric',
                'total_quantity' => 'nullable|numeric',
                'total_gross_amount' => 'nullable|numeric',
                'total_vat_amount' => 'nullable',
                'total_net_amount' => 'nullable',
                'total_purchase_amount_with_cf' => 'nullable',
                'AmountInWords' => 'nullable',
                'new_vat_amount' => 'nullable|numeric',
                'discount_amount' => 'nullable|numeric',
                'name' => 'nullable|string',
                'phone' => 'nullable|string',
                'email' => 'nullable|string',
                'vat_number' => 'nullable|string',
                'address' => 'nullable|string',
                'type' => 'nullable|string',
            ]);
            // dd($validatedData);


            $branch_id = session()->get('selected_branch');


            // Iterate over the item descriptions
            for ($i = 0; $i < count($validatedData['quantity']); $i++) {
                $invoiceData = [
                    'invoice_no' => $newInvoiceNo,
                    'po_no' => $validatedData['po_no'],
                    'lpo_no' => $validatedData['lpo_no'],
                    'lpo_days' => $validatedData['lpo_days'],
                    'term_of_payment' => $validatedData['term_of_payment'],
                    'narrations' => $validatedData['narrations'],
                    'date' => $validatedData['date'],
                    'account_id' => $validatedData['account_id'],
                    'acc_no' => $validatedData['acc_no'],
                    'account_type' => $validatedData['account_type'],
                    'account_name' => $validatedData['name_account'],
                    'item_description' => $validatedData['item_description'][$i] ?? (NULL),
                    'quantity' => $validatedData['quantity'][$i] ?? (NULL),
                    'rate_unit' => $validatedData['rate_unit'][$i] ?? (NULL),
                    'item_unit' => $validatedData['item_unit'][$i] ?? (NULL),
                    'gross_amount' => $validatedData['gross_amount'][$i] ?? (NULL),
                    'vat_percentage' => $validatedData['vat_percentage'][$i] ?? (NULL),
                    'vat_amount' => $validatedData['vat_amount'][$i] ?? (NULL),
                    'net_amount' => $validatedData['net_amount'][$i] ?? (NULL),
                    'total_quantity' => $validatedData['total_quantity'],
                    'total_gross_amount' => $validatedData['total_gross_amount'],
                    'total_vat_amount' => $validatedData['total_vat_amount'],
                    'total_net_amount' => $validatedData['total_net_amount'],
                    'total_purchase_amount_with_cf' => $validatedData['total_purchase_amount_with_cf'],
                    'AmountInWords' => $validatedData['AmountInWords'],
                    'branch_id' => $branch_id,
                    'item_id' => $validatedData['item_id'][$i] ?? (NULL),
                    'warehouse_id' => $validatedData['warehouse_id'][$i] ?? (NULL),
                    'c_f' => $validatedData['c_f'][$i] ?? (NULL),
                    'new_vat_amount' => request()->new_vat_amount_check ? $validatedData['new_vat_amount'] : (NULL),
                    'discount_amount' => request()->discount_amount_check ? $validatedData['discount_amount'] : (NULL),
                ];


                $invoice = AdminPurchaesInvoice::create($invoiceData);


                 // Add expenses for this invoice if expense details are available
                 if (isset($validatedData['expense_id']) && count($validatedData['expense_id']) > 0) {
                    for ($j = 0; $j < count($validatedData['expense_id']); $j++) {
                        PurchaseInvoiceExpense::create([
                            'admin_purchase_invoice_id' => $invoice->id,
                            'expense_type' => $validatedData['expense_type'][$j],
                            'account_name' => $validatedData['account_name'][$j],
                            'expense_id' => $validatedData['expense_id'][$j],
                            'expense_amount' => $validatedData['expense_amount'][$j],
                        ]);
                    }
                }
            }

            flash()->addSuccess('Record Created');
            return redirect()->route('software.purchase_invoices.index');
        } catch (\Throwable $th) {
            flash()->addError($th->getMessage());
            return redirect()->back()->withInput();
        }
    }

    public function show($purchaseInvoice)
    {
        $purchaseInvoice = AdminPurchaesInvoice::withTrashed()->find($purchaseInvoice);
        $purchaseInvoice = $purchaseInvoice->withTrashed()->with('account', 'item', 'warehouse', 'expenses.expense')->where('invoice_no', $purchaseInvoice->invoice_no)->where('branch_id', session()->get('selected_branch'))->get();


        $company = Company::where('user_id', auth()->id())->first();
        $tnc = AdminTermsAndCondition::where(['branch_id'=>session()->get('selected_branch'), 'type'=>'purchase_invoice'])->first();

        return view('crusher.trans_purchase_invoice.view', compact('purchaseInvoice', 'company', 'tnc'));
    }

    public function edit(AdminPurchaesInvoice $purchaseInvoice)
    {

         // Retrieve permission titles from Permission table
         $permissionTitles = Permission::where('branch_id', session()->get('selected_branch'))->pluck('name')->toArray();

         // Filter permission titles using hasBranchPermission function
         $filteredPermissionTitles = [];
         $branchId = session()->get('selected_branch');
         foreach ($permissionTitles as $permissionTitle) {
             if (Branches::hasBranchPermission($branchId, $permissionTitle)) {
                 $filteredPermissionTitles[] = $permissionTitle;
             }
         }

        $items = ItemName::where('branch_id', session()->get('selected_branch'))->get();
        $warehouses = AdminWarehouse::where('branch_id', session()->get('selected_branch'))->get();
        $expenses = Expense::where(['branch_id'=>session()->get('selected_branch'), 'class_of_account_id'=>'Expenses'])
        ->get()
        ->map(function ($item) {
            return $item->setAttribute('table_name', 'App\Expense');
        });
        $otherAccountExpenses = AdminOtherAccount::where(['branch_id'=>session()->get('selected_branch'), 'class_of_account_id'=>'Expenses'])
        ->get()
        ->map(function ($item) {
            return $item->setAttribute('table_name', 'App\AdminOtherAccount');
        });

        $expensesAccount = $expenses->concat($otherAccountExpenses);

        // // Retrieve customers and include both 'account_name' and 'table_name' attributes
        // $customers = Customer::where('branch_id', session()->get('selected_branch'))
        // ->whereIn('name_of_organisation', $filteredPermissionTitles)
        // ->select('*', 'name_of_organisation as account_name')
        // ->get()
        // ->map(function ($item) {
        //     return $item->setAttribute('table_name', 'App\Customer');
        // });

        // // Retrieve suppliers and include both 'account_name' and 'table_name' attributes
        // $suppliers = Supplier::where('branch_id', session()->get('selected_branch'))
        // ->whereIn('name_of_organisation', $filteredPermissionTitles)
        // ->select('*', 'name_of_organisation as account_name')
        // ->get()
        // ->map(function ($item) {
        //     return $item->setAttribute('table_name', 'App\Supplier');
        // });

        // // Retrieve expenses and set 'table_name' to the fully-qualified model class name
        // $salesmens = StaffAndSalesman::where('branch_id', session()->get('selected_branch'))
        // ->whereIn(DB::raw("CONCAT(first_name, ' ', last_name)"), $filteredPermissionTitles)
        // ->select('*', DB::raw("CONCAT(first_name, ' ', last_name) as account_name"))
        // ->get()
        // ->map(function ($item) {
        //     return $item->setAttribute('table_name', 'App\StaffAndSalesman');
        // });

        // // Retrieve expenses and set 'table_name' to the fully-qualified model class name
        // $expenses = Expense::where('branch_id', session()->get('selected_branch'))
        // ->whereIn('account_name', $filteredPermissionTitles)
        // ->get()
        // ->map(function ($item) {
        //     return $item->setAttribute('table_name', 'App\Expense');
        // });

        // // Retrieve other accounts and set 'table_name' to the fully-qualified model class name
        // $otherAccounts = AdminOtherAccount::where('branch_id', session()->get('selected_branch'))
        // ->whereIn('account_name', $filteredPermissionTitles)
        // ->get()
        // ->map(function ($item) {
        //     return $item->setAttribute('table_name', 'App\AdminOtherAccount');
        // });

        $customers = Customer::where('branch_id', session()->get('selected_branch'))
        ->whereIn(DB::raw("REPLACE(name_of_organisation, ' ', '-')"), $filteredPermissionTitles)
        ->select('*', DB::raw("(name_of_organisation) as account_name"))
        ->get()
        ->map(function ($item) {
            return $item->setAttribute('table_name', 'App\Customer');
        });

        $suppliers = Supplier::where('branch_id', session()->get('selected_branch'))
        ->whereIn(DB::raw("REPLACE(name_of_organisation, ' ', '-')"), $filteredPermissionTitles)
        ->select('*', DB::raw("(name_of_organisation) as account_name"))
        ->get()
        ->map(function ($item) {
            return $item->setAttribute('table_name', 'App\Supplier');
        });

        $salesmens = StaffAndSalesman::where('branch_id', session()->get('selected_branch'))
        ->whereIn(DB::raw("CONCAT(REPLACE(first_name, ' ', '-'), '-', REPLACE(last_name, ' ', '-'))"), $filteredPermissionTitles)
        ->select('*', DB::raw("CONCAT(first_name, ' ', last_name) AS account_name"))
        ->get()
        ->map(function ($item) {
            return $item->setAttribute('table_name', 'App\StaffAndSalesman');
        });

        $expenses = Expense::where('branch_id', session()->get('selected_branch'))
        ->whereIn(DB::raw("REPLACE(account_name, ' ', '-')"), $filteredPermissionTitles)
        ->select('*', DB::raw("(account_name)"))
        ->get()
        ->map(function ($item) {
            return $item->setAttribute('table_name', 'App\Expense');
        });

        $otherAccounts = AdminOtherAccount::where('branch_id', session()->get('selected_branch'))
        ->whereIn(DB::raw("REPLACE(account_name, ' ', '-')"), $filteredPermissionTitles)
        ->select('*', DB::raw("(account_name)"))
        ->get()
        ->map(function ($item) {
            $item->setAttribute('table_name', 'App\AdminOtherAccount');
            $item->setAttribute('class_of_account', ($item->class_of_account_id == 1 || $item->class_of_account_id == 3) ? '1,3' : '');
            return $item;
        });

        // Concatenate all collections into a single collection
        $accounts = $customers->concat($expenses)->concat($suppliers)->concat($otherAccounts)->concat($salesmens);

        // $purchaseInvoices = $purchaseInvoice->with('account', 'item', 'warehouse', 'expenses')->where('invoice_no', $purchaseInvoice->invoice_no)->get();

        $purchaseInvoices = AdminPurchaesInvoice::where('branch_id', session()->get('selected_branch'))
        ->with(['account', 'item', 'warehouse', 'expenses'])
        ->where('invoice_no', $purchaseInvoice->invoice_no)
        ->get();

        $periodClosings = AdminPeriodClosing::where('branch_id', session()->get('selected_branch'))->pluck('to')->toArray();


        // dd($purchaseInvoices);
        return view('crusher.trans_purchase_invoice.edit', compact('items', 'warehouses', 'expensesAccount', 'accounts', 'purchaseInvoices','periodClosings'));
    }
    public function update(Request $request, AdminPurchaesInvoice $purchaseInvoice)
    {
        // dd($request);
        try {
            $validatedData = $request->validate([
                'invoice_no' => 'nullable|string',
                'po_no' => 'nullable|string',
                'lpo_no' => 'nullable|string',
                'lpo_days' => 'nullable|string',
                'term_of_payment' => 'nullable|string',
                'narrations' => 'nullable|string',
                'date' => 'nullable|date',
                'account_id' => 'nullable',
                'acc_no' => 'nullable',
                'account_id' => 'nullable|integer',
                'account_type' => 'nullable',
                'name_account' => 'nullable',
                'item_description' => 'nullable|array',
                'item_description.*' => 'nullable|string',
                'item_id' => 'nullable|array',
                'item_id.*' => 'nullable|exists:item_names,id',
                'item_unit' => 'nullable|array',
                'item_unit.*' => 'nullable|string',
                'warehouse_id' => 'nullable|array',
                'warehouse_id.*' => 'nullable|exists:admin_warehouses,id',
                'quantity' => 'nullable|array',
                'quantity.*' => 'nullable|numeric',
                'rate_unit' => 'nullable|array',
                'rate_unit.*' => 'nullable|numeric',
                'gross_amount' => 'nullable|array',
                'gross_amount.*' => 'nullable|numeric',
                'vat_percentage' => 'nullable|array',
                'vat_percentage.*' => 'nullable|numeric',
                'vat_amount' => 'nullable|array',
                'vat_amount.*' => 'nullable|numeric',
                'net_amount' => 'nullable|array',
                'net_amount.*' => 'nullable|numeric',
                'c_f' => 'nullable|array',
                'c_f.*' => 'nullable|numeric',
                'expense_type' => 'nullable|array',
                'expense_type.*' => 'nullable',
                'account_name' => 'nullable|array',
                'account_name.*' => 'nullable',
                'expense_id' => 'nullable|array',
                'expense_id.*' => 'nullable|integer',
                'expense_amount' => 'nullable|array',
                'expense_amount.*' => 'nullable|numeric',
                'total_quantity' => 'nullable|numeric',
                'total_gross_amount' => 'nullable|numeric',
                'total_vat_amount' => 'nullable',
                'total_net_amount' => 'nullable',
                'total_purchase_amount_with_cf' => 'nullable',
                'AmountInWords' => 'nullable',
                'new_vat_amount' => 'nullable|numeric',
                'discount_amount' => 'nullable|numeric',
            ]);

            $branch_id = session()->get('selected_branch');
            $purchaseInvoices = $purchaseInvoice->with('account', 'item', 'warehouse', 'expenses')->where('branch_id', session()->get('selected_branch'))->where('invoice_no', $purchaseInvoice->invoice_no)->forceDelete();

            // Iterate over the item descriptions
            for ($i = 0; $i < count($validatedData['quantity']); $i++) {
                $invoiceData = [
                    'invoice_no' => $validatedData['invoice_no'],
                    'po_no' => $validatedData['po_no'],
                    'lpo_no' => $validatedData['lpo_no'],
                    'lpo_days' => $validatedData['lpo_days'],
                    'term_of_payment' => $validatedData['term_of_payment'],
                    'narrations' => $validatedData['narrations'],
                    'date' => $validatedData['date'],
                    'account_id' => $validatedData['account_id'],
                    'acc_no' => $validatedData['acc_no'],
                    'account_type' => $validatedData['account_type'],
                    'account_name' => $validatedData['name_account'],
                    'item_description' => $validatedData['item_description'][$i] ?? (NULL),
                    'quantity' => $validatedData['quantity'][$i] ?? (NULL),
                    'rate_unit' => $validatedData['rate_unit'][$i] ?? (NULL),
                    'item_unit' => $validatedData['item_unit'][$i] ?? (NULL),
                    'gross_amount' => $validatedData['gross_amount'][$i] ?? (NULL),
                    'vat_percentage' => $validatedData['vat_percentage'][$i] ?? (NULL),
                    'vat_amount' => $validatedData['vat_amount'][$i] ?? (NULL),
                    'net_amount' => $validatedData['net_amount'][$i] ?? (NULL),
                    'total_quantity' => $validatedData['total_quantity'],
                    'total_gross_amount' => $validatedData['total_gross_amount'],
                    'total_vat_amount' => $validatedData['total_vat_amount'],
                    'total_net_amount' => $validatedData['total_net_amount'],
                    'total_purchase_amount_with_cf' => $validatedData['total_purchase_amount_with_cf'],
                    'AmountInWords' => $validatedData['AmountInWords'],
                    'branch_id' => $branch_id,
                    'item_id' => $validatedData['item_id'][$i] ?? (NULL),
                    'warehouse_id' => $validatedData['warehouse_id'][$i] ?? (NULL),
                    'c_f' => $validatedData['c_f'][$i] ?? (NULL),
                    'new_vat_amount' => request()->new_vat_amount_check ? $validatedData['new_vat_amount'] : (NULL),
                    'discount_amount' => request()->discount_amount_check ? $validatedData['discount_amount'] : (NULL),
                ];

                $invoice[] = AdminPurchaesInvoice::create($invoiceData);
            }
            // dd($invoice);
            if (isset($validatedData['expense_id']))
            {
                PurchaseInvoiceExpense::where('branch_id', session()->get('selected_branch'))->where('admin_purchase_invoice_id',$purchaseInvoice->id)->forceDelete();
                // Add expenses for this invoice
                for ($j = 0; $j < count($validatedData['expense_id']); $j++) {
                    PurchaseInvoiceExpense::create([
                        'admin_purchase_invoice_id' => $invoice[0]->id,
                        'expense_type' => $validatedData['expense_type'][$j],
                        'account_name' => $validatedData['account_name'][$j],
                        'expense_id' => $validatedData['expense_id'][$j],
                        'expense_amount' => $validatedData['expense_amount'][$j],
                    ]);
                }
            }

            flash()->addSuccess('Record Updated');
            return redirect()->route('software.purchase_invoices.index');
        } catch (\Throwable $th) {
            // dd($th, $purchaseInvoice);
            flash()->addError($th->getMessage());
            return redirect()->back()->withInput();
        }
    }

    /**
     * Remove the specified resource from storage.
     *
     * @param  \App\Quotation  $quotation
     * @return \Illuminate\Http\Response
     */
    public function destroy(Quotation $quotation)
    {
        //
    }

    public function softDelete($purchaseInvoice)
    {
        $purchaseInvoice = AdminPurchaesInvoice::findOrFail($purchaseInvoice);
        $purchaseInvoice->where('branch_id', session()->get('selected_branch'))->where('invoice_no', $purchaseInvoice->invoice_no)->delete();
        flash()->addSuccess('Success');
        return redirect()->route('crusher.purchase_invoices.index');
    }

    function buyer(Request $request)
    {
        // return $request->all();
        try {
            unset($request['buyer_account_id']);
            unset($request['_token']);
            OtherAccountBuyer::firstOrCreate(['type'=>$request->type, 'invoice_no'=>$request->invoice_no],$request->all());
            return response()->json(['status'=>true]);
        } catch (\Throwable $th) {
            return response()->json(['status'=>false, 'error'=>$th->getMessage()]);
        }
    }

    function buyerExists(Request $request)
    {
        // return $request->all();
        try {
            $buyer = OtherAccountBuyer::where(['type'=>$request->type, 'invoice_no'=>$request->invoice_no])->exists();
            return response()->json(['status'=>$buyer]);
        } catch (\Throwable $th) {
            return response()->json(['status'=>false, 'error'=>$th->getMessage()]);
        }
    }

    function buyerDelete(Request $request)
    {
        // return $request->all();
        try {
            $buyer = OtherAccountBuyer::where(['type'=>$request->type, 'invoice_no'=>$request->invoice_no])->delete();
            return response()->json(['status'=>$buyer]);
        } catch (\Throwable $th) {
            return response()->json(['status'=>false, 'error'=>$th->getMessage()]);
        }
    }
}
