<?php

namespace App;

use Illuminate\Database\Eloquent\Model;
use Illuminate\Database\Eloquent\Relations\HasMany;

class Branches extends Model
{
    protected $table = 'branches';

    // RELATIONS

    public function company()
    {
        return $this->belongsTo(Company::class, 'company_id', 'id');
    }

    public function permissions()
    {
        return $this->hasMany(BranchPermission::class, 'branch_id', 'id');
    }

    public function vouchers()
    {
        return $this->hasMany(ReceivedVoucher::class, 'branch_id', 'id');
    }

    public static function hasBranchPermission($branchId, $permissionTitle)
    {
        $permission = Permission::where('name', $permissionTitle)->first();

        if (!$permission) {
            return false;
        }

        // if (auth()->user()->role=='subadmin') {
        // return AssignedPermission::whereHas('branchPermission.branch', function($q)use($branchId){$q->where('id',$branchId);})
        // ->whereHas('branchPermission.permission', function($q)use($permission){$q->where('id',$permission->id);})
        // ->exists();
        // }
        // return BranchPermission::where('branch_id', $branchId)
        //     ->where('permission_id', $permission->id)
        //     ->exists();

        if (auth()->user()->role == 'subadmin') {
            $user_id = auth()->user()->id;

            return AssignedPermission::where('subadmin_id', $user_id)
                ->whereHas('branchPermission.branch', function($q) use ($branchId) {
                    $q->where('id', $branchId);
                })
                ->whereHas('branchPermission.permission', function($q) use ($permission) {
                    $q->where('id', $permission->id);
                })
                ->exists();
        }
        return BranchPermission::where('branch_id', $branchId)
        ->where('permission_id', $permission->id)
        ->exists();

    }

}


