<?php

namespace App\Http\Controllers;

use App\AdminItemGroup;
use App\Branches;
use Illuminate\Database\Eloquent\SoftDeletes;
use Illuminate\Http\Request;

class AdminItemGroupController extends Controller
{
    use SoftDeletes;

    public function index()
    {
        $itemGroups = AdminItemGroup::where('branch_id', session()->get('selected_branch'))->get();
        return view('crusher.itemgroup_master.index', compact('itemGroups'));
    }

    public function create()
    {
        return view('crusher.itemgroup_master.create');
    }

    function store(Request $request)
    {
        try {
            $validatedData = $request->validate([
                'name' => 'required|array',
                'name.*' => 'required|string',
            ]);

            foreach ($validatedData['name'] as $key => $name) {
                AdminItemGroup::create(['branch_id'=>session()->get('selected_branch'), 'name'=>$name]);
            }
            
    
            flash()->addSuccess('Record Created');
            return redirect()->route('crusher.item_groups.index');
        } catch (\Throwable $th) {
            flash()->addError($th->getMessage());
            return redirect()->back()->withInput();
        }
    }

    public function edit($itemGroup)
    {
        $itemGroup = AdminItemGroup::find($itemGroup);
        return view('crusher.itemgroup_master.edit', compact('itemGroup'));
    }

    function update(Request $request, $itemGroup)
    {
        try {
            $validatedData = $request->validate([
                'name' => 'required|string',
            ]);
    
            AdminItemGroup::find($itemGroup)->update($validatedData);
    
            flash()->addSuccess('Record Updated');
            return redirect()->route('crusher.item_groups.index');
        } catch (\Throwable $th) {
            flash()->addError($th->getMessage());
            return redirect()->back()->withInput();
        }
    }

    public function show($itemGroup)
    {
        $itemGroup = AdminItemGroup::find($itemGroup);
        return view('crusher.itemgroup_master.view', compact('itemGroup'));
    }

    public function softDelete($itemGroup)
    {
        $itemGroup = AdminItemGroup::findOrFail($itemGroup);
        $itemGroup->delete();
        flash()->addSuccess('Success');
        return redirect()->route('crusher.item_groups.index');
    }

    // TRANSFER
    function createTransfer()
    {
        $itemGroups = AdminItemGroup::where('branch_id', session()->get('selected_branch'))->get();
        $branches = Branches::whereHas('company', function($q){$q->where('user_id', auth()->id());})->where('id','!=',session()->get('selected_branch'))->get();
        return view('crusher.itemgroup_master.transfer_create', compact('itemGroups', 'branches'));
    }

    function transfer(Request $request)
    {
        try {
            $validatedData = $request->validate([
                'branch' => 'required',
                'groups' => 'required|array',
                'groups.*' => 'required|exists:admin_item_groups,id',
            ]);

            foreach ($validatedData['groups'] as $key => $group) {
                $existingGroup = AdminItemGroup::find($group);
                AdminItemGroup::create(['branch_id'=>$validatedData['branch'], 'name'=>$existingGroup->name]);
            }
            
    
            flash()->addSuccess('Success');
            return redirect()->route('crusher.item_groups.index');
        } catch (\Throwable $th) {
            flash()->addError($th->getMessage());
            return redirect()->back()->withInput();
        }
    }
}
