<?php

namespace App\Http\Controllers\MainMenu;

use App\ApprovalVoucher;
use App\ApprovalVoucherItem;
use App\Country;
use App\DepositWithdrawVoucher;
use App\Http\Controllers\Controller;
use App\LocationPermission;
use App\LocationPermissionTrack;
use App\PurchaseInvoice;
use App\PurchaseInvoiceItem;
use App\PurchaseReturnVoucher;
use App\PurchaseReturnVoucherItem;
use App\SaleInvoice;
use App\SaleInvoiceItem;
use App\SalesReturnVoucher;
use App\SalesReturnVoucherItem;
use Illuminate\Http\Request;
use App\Services\Utils;
use App\WarehouseStockTransfer;
use App\WarehouseStockTransferItem;
use Illuminate\Support\Facades\Auth;

class SearchKeywordNumberController extends Controller
{
    /**
     * Display a listing of the resource.
     *
     * @return \Illuminate\Http\Response
     */

    public function search()
    {
        $user = Auth::user();
        if (!$user->can('search-keyword-number-permission')) {
            abort(403, 'Unauthorized action.');
        }

        if(Auth::user()->hasRole('user')){
            $permission = LocationPermission::Where('user_id', Auth::user()->id)->get();

            $option = LocationPermissionTrack::Where('user_id', Auth::user()->id)->first();
            $permission_option = $option->permission_option;

            foreach ($permission as $perm) {
                $countries[] =  Country::Where('id', $perm->country_id)->first();
            }
            $countries = array_unique($countries);

            return view('user.main_menu.search_keyword_number.index', compact('countries', 'permission_option'));

        }
        else{
            $countries = Country::all();
            $permission_option = "all_countries_all_locations";
            return view('user.main_menu.search_keyword_number.index', compact('countries', 'permission_option'));
        }
    }

    public function searchResult(Request $request)
    {
        $user = Auth::user();
        if (!$user->can('search-keyword-number-permission')) {
            abort(403, 'Unauthorized action.');
        }

        $country_id = $request->country;
        $location_id = $request->location;
        $search = $request->search;

        $approval_voucher_narrations_ids = [];
        $approval_voucher_imei_ids = [];
        $approval_voucher_result = [];

        $sales_invoice_narrations_ids = [];
        $sales_invoice_imei_ids = [];
        $sales_invoice_result = [];

        $purchase_invoice_narrations_ids = [];
        $purchase_invoice_imei_ids = [];
        $purchase_invoice_result = [];

        $sales_return_invoice_narrations_ids = [];
        $sales_return_invoice_imei_ids = [];
        $sales_return_invoice_result = [];

        $purchase_return_invoice_narrations_ids = [];
        $purchase_return_invoice_imei_ids = [];
        $purchase_return_invoice_result = [];

        $warehouse_stock_transfer_imei_ids = [];
        $warehouse_stock_transfer_result = [];

        Utils::setLocation($location_id);

        //Approval Voucher

        $approval_voucher_narrations = ApprovalVoucher::withTrashed()->Where( 'narrations', 'LIKE', '%' . $search . '%' )
        ->orWhere( 'buyer_name', 'LIKE', '%' . $search . '%' )->orWhere( 'buyer_mobile', 'LIKE', '%' . $search . '%' )->get();

        $approval_voucher_narrations = $approval_voucher_narrations->where('location_id', $location_id);
        foreach ($approval_voucher_narrations as $item) {
            $approval_voucher_narrations_ids[] = $item->id;
        }

        $approval_voucher_imei = ApprovalVoucherItem::Where('imei_serial_no', $search)->get();

        foreach ($approval_voucher_imei as $item) {
            $approval_voucher_imei_ids[] = $item->approval_voucher_id;
        }

        $approval_merge = array_merge($approval_voucher_narrations_ids, $approval_voucher_imei_ids);
        $approval_voucher_merge = array_unique($approval_merge);

        foreach ($approval_voucher_merge as $id) {
            $approval_voucher_result[] = ApprovalVoucher::withTrashed()->with('account')->Where('id', $id)->first();
        }

        //SalesInvoice

        $sales_invoice_narrations = SaleInvoice::withTrashed()->Where( 'narrations', 'LIKE', '%' . $search . '%' )
        ->orWhere( 'buyer_name', 'LIKE', '%' . $search . '%' )->orWhere( 'buyer_mobile', 'LIKE', '%' . $search . '%' )->get();

        $sales_invoice_narrations = $sales_invoice_narrations->where('location_id', $location_id);
        foreach ($sales_invoice_narrations as $item) {
            $sales_invoice_narrations_ids[] = $item->id;
        }

        $sales_invoice_imei = SaleInvoiceItem::Where('imei_serial_no', $search)->with(['invoice' => function($q) use ($location_id) {
            $q->where('location_id', $location_id);}])->get();

        foreach ($sales_invoice_imei as $item) {
            if($item->invoice){
                $sales_invoice_imei_ids[] = $item->sales_invoice_id;
            }
        }

        $sales_merge = array_merge($sales_invoice_narrations_ids, $sales_invoice_imei_ids);
        $sales_invoice_merge = array_unique($sales_merge);

        foreach ($sales_invoice_merge as $id) {
            $sales_invoice_result[] = SaleInvoice::withTrashed()->with('account')->Where('id', $id)->first();
        }

        //PurchaseInvoice

        $purchase_invoice_narrations = PurchaseInvoice::withTrashed()->Where('invoice_no', '!=', 0)->Where( 'narrations', 'LIKE', '%' . $search . '%' )
        ->orWhere( 'buyer_name', 'LIKE', '%' . $search . '%' )->orWhere( 'buyer_mobile', 'LIKE', '%' . $search . '%' )->get();

        $purchase_invoice_narrations = $purchase_invoice_narrations->where('location_id', $location_id);
        foreach ($purchase_invoice_narrations as $item) {
            $purchase_invoice_narrations_ids[] = $item->id;
        }

        $purchase_invoice_imei = PurchaseInvoiceItem::Where('imei_serial_no', $search)->with(['invoice' => function($q) use ($location_id) {
            $q->where('location_id', $location_id);}])->get();

        foreach ($purchase_invoice_imei as $item) {
            if($item->invoice){
                $purchase_invoice_imei_ids[] = $item->invoice->id;
            }
        }

        $purchase_merge = array_merge($purchase_invoice_narrations_ids, $purchase_invoice_imei_ids);
        $purchase_invoice_merge = array_unique($purchase_merge);

        foreach ($purchase_invoice_merge as $id) {
            $purchase_invoice_result[] = PurchaseInvoice::withTrashed()->with('account','items')->Where('id', $id)->first();
        }
        $sales_items = [];
        foreach ($purchase_invoice_result as $invoice) {
            foreach ($invoice->items as $item) {
                $sales_items[] = SaleInvoiceItem::where('imei_serial_no', $item->imei_serial_no)->first();
            }

        }

        //SalesReturn

        $sales_return_invoice_narrations = SalesReturnVoucher::withTrashed()->Where( 'narrations', 'LIKE', '%' . $search . '%' )
        ->orWhere( 'buyer_name', 'LIKE', '%' . $search . '%' )->orWhere( 'buyer_mobile', 'LIKE', '%' . $search . '%' )->get();

        $sales_return_invoice_narrations = $sales_return_invoice_narrations->where('location_id', $location_id);
        foreach ($sales_return_invoice_narrations as $item) {
            $sales_return_invoice_narrations_ids[] = $item->id;
        }

        $sales_return_invoice_imei = SalesReturnVoucherItem::Where('imei_serial_no', $search)->with(['invoice' => function($q) use ($location_id) {
            $q->where('location_id', $location_id);}])->get();

        foreach ($sales_return_invoice_imei as $item) {
            if($item->invoice){
                $sales_return_invoice_imei_ids[] = $item->sales_return_voucher_id;
            }
        }

        $sales_return_merge = array_merge($sales_return_invoice_narrations_ids, $sales_return_invoice_imei_ids);
        $sales_return_invoice_merge = array_unique($sales_return_merge);

        foreach ($sales_return_invoice_merge as $id) {
            $sales_return_invoice_result[] = SalesReturnVoucher::withTrashed()->with('account')->Where('id', $id)->first();
        }

        //PurchaseReturn

        $purchase_return_invoice_narrations = PurchaseReturnVoucher::withTrashed()->Where( 'narrations', 'LIKE', '%' . $search . '%' )
        ->orWhere( 'buyer_name', 'LIKE', '%' . $search . '%' )->orWhere( 'buyer_mobile', 'LIKE', '%' . $search . '%' )->get();

        $purchase_return_invoice_narrations = $purchase_return_invoice_narrations->where('location_id', $location_id);
        foreach ($purchase_return_invoice_narrations as $item) {
            $purchase_return_invoice_narrations_ids[] = $item->id;
        }

        $purchase_return_invoice_imei = PurchaseReturnVoucherItem::Where('imei_serial_no', $search)->with(['invoice' => function($q) use ($location_id) {
            $q->where('location_id', $location_id);}])->get();

        foreach ($purchase_return_invoice_imei as $item) {
            if($item->invoice){
                $purchase_return_invoice_imei_ids[] = $item->purchase_return_voucher_id;
            }
        }

        $purchase_return_merge = array_merge($purchase_return_invoice_narrations_ids, $purchase_return_invoice_imei_ids);
        $purchase_return_invoice_merge = array_unique($purchase_return_merge);

        foreach ($purchase_return_invoice_merge as $id) {
            $purchase_return_invoice_result[] = PurchaseReturnVoucher::withTrashed()->with('account')->Where('id', $id)->first();
        }

        //DepositAndWithdraw

        $deposit_and_withdraw = DepositWithdrawVoucher::withTrashed()->where('location_id', $location_id)->Where( 'narrations', 'LIKE', '%' . $search . '%' )->get();

        //WareHouseStockTransfer

        $warehouse_stock_transfer_imei = WarehouseStockTransferItem::withTrashed()->Where('imei_serial_no', $search)->get();

        foreach ($warehouse_stock_transfer_imei as $item) {
            $warehouse_stock_transfer_imei_ids[] = $item->warehouse_stock_transfer_id;
        }

        foreach ($warehouse_stock_transfer_imei_ids as $id) {
            $warehouse_stock_transfer_result[] = WarehouseStockTransfer::withTrashed()->Where('id', $id)->first();
        }

        return view('user.main_menu.search_keyword_number.search_result', compact('search', 'sales_items','approval_voucher_result','sales_invoice_result','purchase_invoice_result','sales_return_invoice_result','purchase_return_invoice_result','deposit_and_withdraw','warehouse_stock_transfer_result'));
    }

    /**
     * Show the form for creating a new resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function create()
    {
        //
    }

    /**
     * Store a newly created resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @return \Illuminate\Http\Response
     */
    public function store(Request $request)
    {
        //
    }

    /**
     * Display the specified resource.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function show($id)
    {
        //
    }

    /**
     * Show the form for editing the specified resource.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function edit($id)
    {
        //
    }

    /**
     * Update the specified resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function update(Request $request, $id)
    {
        //
    }

    /**
     * Remove the specified resource from storage.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function destroy($id)
    {
        //
    }
}
