<?php

namespace App\Http\Controllers\Master\AccountMaster;

use Illuminate\Http\Request;
use App\Http\Controllers\Controller;

use Auth;
use DB;
use Storage;
use Session;
use Carbon\Carbon;
use App\Services\Utils;
// use App\OtherAccount;
use App\AccountMaster;
use App\AccountMastersAccountsDetail;
use App\AllCountries;
use App\User;
use App\Location;
use App\LocationPermission;
use App\ClassOfAccount;
use App\AccountMasterTrack;
use Illuminate\Support\Facades\Hash;

class OtherAccountController extends Controller
{

    protected $utils;

    public function __construct(Utils $utils)
    {
        $this->utils = $utils;
    }

    /**
     * Display a listing of the resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function index()
    {
        $user = Auth::user();
        if (!$user->can('other-account-master')) {
            abort(403, 'Unauthorized action.');
        }

        $location_id = $this->getLocationId();
        $other_accounts = AccountMaster::Where('location_id', $location_id)->Where('account_master_type', "Other-Account")->orderBy('created_at','DESC')->paginate(100);
        $count = [];
        foreach ($other_accounts as $account) {
            $account_detail = AccountMastersAccountsDetail::Where('account_master_id', $account->id)->where('voucher_type', '!=' ,"opening balance")->get();
            $count[] = count($account_detail);
        }
        return view('user.masters.account_masters.other_accounts.index', compact('other_accounts', 'count'));
    }

    /**
     * Show the form for creating a new resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function create()
    {
        $user = Auth::user();
        if (!$user->can('other-account-master')) {
            abort(403, 'Unauthorized action.');
        }

        $location_id = $this->getLocationId();
        $country = Location::where('id', $location_id)->first();
        $class_of_accounts = ClassOfAccount::all();
        $users = LocationPermission::with('users')->where('country_id', $country->country_id)->where('location_id', $location_id)->where('user_id', '!=', $user->id)->get();
        $all_countries = AllCountries::all();
        return view('user.masters.account_masters.other_accounts.create', compact('all_countries', 'class_of_accounts', 'users'));
    }

    /**
     * Store a newly created resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @return \Illuminate\Http\Response
     */
    public function store(Request $request)
    {
        $location_id = $this->getLocationId();
        $other_account = AccountMaster::Where('name_of_organisation', $request->name_of_organisation)->Where('account_master_type', "Other-Account")->where('location_id', $location_id)->first();
        if ($other_account) {
            Session::flash('alert-danger', 'This Other Account Already Exists');
            return redirect()->back();
        }
        else
        $other_account = new AccountMaster();
        $other_account->location_id = $location_id;
        $other_account->account_master_type = "Other-Account";
        if ($request->transactions_password != "") {
            $other_account->transactions_password = Hash::make($request->transactions_password);
        }
        $other_account->opening_balance = $request->opening_balance;
        $other_account->debit_credit = $request->debit_credit;
        if ($request->report_password != "") {
            $other_account->report_password = Hash::make($request->report_password);
        }
        $other_account->date = $request->date;
        $other_account->name_of_organisation = $request->name_of_organisation;
        $other_account->class_of_account_id = $request->class_of_account_id;
        // $other_account->show_user = $request->show_user;
        $other_account->description = $request->description;
        $other_account->save();
        $current_other_account_id = $other_account->id;

        if(isset($request->show_user)){
            foreach ($request->show_user as $key => $request->show_user) {
                $account_master_track = new AccountMasterTrack();
                $account_master_track->user_id = $request->show_user;
                $account_master_track->account_master_id = $current_other_account_id;
                $account_master_track->account_master_name = "Other-Account";
                $account_master_track->save();
            }
        }

        if(Auth::user()->hasRole('super-admin'))
        {
            $account_master_track = new AccountMasterTrack();
            $account_master_track->user_id = Auth::user()->id;
            $account_master_track->account_master_id = $current_other_account_id;
            $account_master_track->account_master_name = "Other-Account";
            $account_master_track->save();
        }else{
            $admins = User::role('super-admin')->get();
            foreach ($admins as $key => $admin) {
                $account_master_track = new AccountMasterTrack();
                $account_master_track->user_id = $admin->id;
                $account_master_track->account_master_id = $current_other_account_id;
                $account_master_track->account_master_name = "Other-Account";
                $account_master_track->save();
            }
            $account_master_track = new AccountMasterTrack();
            $account_master_track->user_id = Auth::user()->id;
            $account_master_track->account_master_id = $current_other_account_id;
            $account_master_track->account_master_name = "Other-Account";
            $account_master_track->save();
        }

        if ($request->opening_balance != "") {
            $account = new AccountMastersAccountsDetail;
            $account->account_master_id = $other_account->id;
            if ($request->debit_credit == "debit") {
                $account->debit = $request->opening_balance;
                $account->credit = 0;
            }elseif ($request->debit_credit == "credit") {
                $account->debit = 0;
                $account->credit = $request->opening_balance;
            }
            $account->date = $request->date;
            $account->voucher_type = "opening balance";
            $account->save();
        }

        Session::flash('alert-success', 'Other Account created successfully');
        return redirect('/masters/account-masters/other-accounts');
    }

    /**
     * Display the specified resource.
     *
     * @param  \App\OtherAccount  $other_account
     * @return \Illuminate\Http\Response
     */
    public function show(OtherAccount $other_account)
    {
        //
    }

    /**
     * Show the form for editing the specified resource.
     *
     * @param  \App\OtherAccount  $other_account
     * @return \Illuminate\Http\Response
     */
    public function edit($other_account_id)
    {
        $user = Auth::user();
        if (!$user->can('other-account-master')) {
            abort(403, 'Unauthorized action.');
        }

        $location_id = $this->getLocationId();
        $country = Location::where('id', $location_id)->first();
        $class_of_accounts = ClassOfAccount::all();
        $users = LocationPermission::with('users')->where('country_id', $country->country_id)->where('location_id', $location_id)->where('user_id', '!=', $user->id)->get();
        $account_master_track = AccountMasterTrack::Where('account_master_id', $other_account_id)
                                                    ->Where('account_master_name', "Other-Account")->pluck('user_id');
        $all_countries = AllCountries::all();
        $other_account = AccountMaster::Where('id', $other_account_id)
                                        ->Where('account_master_type', "Other-Account")->first();
        return view('user.masters.account_masters.other_accounts.edit', compact('other_account', 'all_countries', 'class_of_accounts', 'users', 'account_master_track'));
    }

    /**
     * Update the specified resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @param  \App\OtherAccount  $other_account
     * @return \Illuminate\Http\Response
     */
    public function update(Request $request, $other_account_id)
    {
        $other_account = AccountMaster::Where('id', $other_account_id)
                                        ->Where('account_master_type', "Other-Account")->first();
        $previous_opening_balance = (float)$other_account->opening_balance;
        $current_opening_balance = (float)$request->opening_balance;
        if ($request->transactions_password != "") {
            $other_account->transactions_password = Hash::make($request->transactions_password) ? Hash::make($request->transactions_password) : $other_account->transactions_password;
        }
        $other_account->opening_balance = $request->opening_balance;
        if ($request->report_password != "") {
            $other_account->report_password = Hash::make($request->report_password) ? Hash::make($request->report_password) : $other_account->report_password;
        }
        $other_account->date = $request->date;
        $other_account->name_of_organisation = $request->name_of_organisation;
        $other_account->class_of_account_id = $request->class_of_account_id;
        // $other_account->show_user = $request->show_user;
        $other_account->description = $request->description;
        $other_account->save();

        $account_master_track = AccountMasterTrack::Where('account_master_id', $other_account_id)
                                                    ->Where('account_master_name', "Other-Account")->get();
        foreach ($account_master_track as $item) {
            $item->delete();
        }
        if(isset($request->show_user)){
            foreach ($request->show_user as $key => $request->show_user) {
                $account_master_track = new AccountMasterTrack();
                $account_master_track->user_id = $request->show_user;
                $account_master_track->account_master_id = $other_account_id;
                $account_master_track->account_master_name = "Other-Account";
                $account_master_track->save();
            }
        }

        if(Auth::user()->hasRole('super-admin'))
        {
            $account_master_track = new AccountMasterTrack();
            $account_master_track->user_id = Auth::user()->id;
            $account_master_track->account_master_id = $other_account_id;
            $account_master_track->account_master_name = "Other-Account";
            $account_master_track->save();
        }else{
            $admins = User::role('super-admin')->get();
            foreach ($admins as $key => $admin) {
                $account_master_track = new AccountMasterTrack();
                $account_master_track->user_id = $admin->id;
                $account_master_track->account_master_id = $other_account_id;
                $account_master_track->account_master_name = "Other-Account";
                $account_master_track->save();
            }
            $account_master_track = new AccountMasterTrack();
            $account_master_track->user_id = Auth::user()->id;
            $account_master_track->account_master_id = $other_account_id;
            $account_master_track->account_master_name = "Other-Account";
            $account_master_track->save();
        }

        // if ($request->opening_balance != "") {
        //     $account = AccountMastersAccountsDetail::Where('account_master_id', $request->account_id)->first();

        //     if ($account != "") {
        //         if ($previous_opening_balance > $current_opening_balance) {
        //             $diff = $previous_opening_balance - $current_opening_balance;
        //             $account->credit =  (float)$account->credit - (float)$diff;
        //         }
        //         elseif ($previous_opening_balance < $current_opening_balance) {
        //             $diff = $current_opening_balance - $previous_opening_balance;
        //             $account->credit = (float)$account->credit + (float)$diff;
        //         }

        //         $account->save();

        //     }else{
        //         $account = new AccountMastersAccountsDetail();
        //         $account->account_master_id = $other_account->id;
        //         $account->debit = 0;
        //         $account->credit = $request->opening_balance;
        //         $account->save();
        //     }

        // }

        Session::flash('alert-success', 'Other Account updated successfully');
        return redirect('/masters/account-masters/other-accounts');
    }

    /**
     * Remove the specified resource from storage.
     *
     * @param  \App\OtherAccount  $other_account
     * @return \Illuminate\Http\Response
     */
    public function destroy($other_account_id)
    {
        $other_account = AccountMaster::Where('id', $other_account_id)->Where('account_master_type', "Other-Account")->first();
        $other_account->delete();

        $account_master_track = AccountMasterTrack::Where('account_master_id', $other_account_id)
                                                    ->Where('account_master_name', "Other-Account")->get();
        foreach ($account_master_track as $item) {
            $item->delete();
        }

        Session::flash('alert-success', 'Other Account deleted successfully');
        return redirect('/masters/account-masters/other-accounts');
    }

    public function softdelete($other_account_id)
    {
        $other_account = AccountMaster::Where('id', $other_account_id)->Where('account_master_type', "Other-Account")->first();
        $other_account->forceDelete();

        $account_master_track = AccountMasterTrack::Where('account_master_id', $other_account_id)
                                                    ->Where('account_master_name', "Other-Account")->get();
        foreach ($account_master_track as $item) {
            $item->forceDelete();
        }

        Session::flash('alert-success', 'Other Account deleted successfully');
        return redirect('/masters/account-masters/other-accounts');
    }
}
