<?php

namespace App\Http\Controllers\Master;

use Illuminate\Http\Request;
use App\Http\Controllers\Controller;
use App\ItemName;
use Auth;
use DB;
use Storage;
use Session;
use Carbon\Carbon;
use App\Services\Utils;
use App\Services\Slug;
use App\ItemUnit;

class ItemUnitController extends Controller
{

    protected $utils;
    protected $slug;

    public function __construct(Utils $utils, Slug $slug)
    {
        $this->utils = $utils;
        $this->slug = $slug;
    }

    /**
     * Display a listing of the resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function index()
    {
        $user = Auth::user();
        if (!$user->can('item-unit-master')) {
            abort(403, 'Unauthorized action.');
        }

        $location_id = $this->getLocationId();
        $item_units = ItemUnit::Where('location_id', $location_id)->orderBy('created_at','DESC')->paginate(100);
        return view('user.masters.item_units.index', compact('item_units'));
    }

    /**
     * Show the form for creating a new resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function create()
    {
        $user = Auth::user();
        if (!$user->can('item-unit-master')) {
            abort(403, 'Unauthorized action.');
        }

        return view('user.masters.item_units.create');
    }

    /**
     * Store a newly created resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @return \Illuminate\Http\Response
     */
    public function store(Request $request)
    {
        $location_id = $this->getLocationId();

        foreach ($request->unit as $key => $unit) {
            $item_unit = new ItemUnit();
            $item_unit->location_id = $location_id;
            $item_unit->unit = $request->unit[$key];
            $item_unit->save();
        }
        Session::flash('alert-success', 'Item Unit created successfully');
        return redirect('/masters/item-units');
    }

    /**
     * Display the specified resource.
     *
     * @param  \App\ItemUnit  $item_unit
     * @return \Illuminate\Http\Response
     */
    public function show(ItemUnit $item_unit)
    {
        //
    }

    /**
     * Show the form for editing the specified resource.
     *
     * @param  \App\ItemUnit  $item_unit
     * @return \Illuminate\Http\Response
     */
    public function edit($item_unit_id)
    {
        $user = Auth::user();
        if (!$user->can('item-unit-master')) {
            abort(403, 'Unauthorized action.');
        }

        $item_unit = ItemUnit::find($item_unit_id);
        return view('user.masters.item_units.edit', compact('item_unit'));
    }

    /**
     * Update the specified resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @param  \App\ItemUnit  $item_unit
     * @return \Illuminate\Http\Response
     */
    public function update(Request $request, $item_unit_id)
    {
        $location_id = $this->getLocationId();
        $unit_name = ItemUnit::find($item_unit_id);
        $previous_name = $unit_name->unit;
        if ($previous_name != $request->unit) {
            $item = ItemUnit::Where('unit', $request->unit)->where('location_id', $location_id)->first();
            if ($item) {
                Session::flash('alert-danger', '' .$request->unit. ' Unit Name Already Exists');
                return redirect('/masters/item-units');
            }
            $unit_name->unit = $request->unit;

            $unit_name->save();

            Session::flash('alert-success', 'Item Unit Name updated successfully');
            return redirect('/masters/item-units');

        }
        else
        {
        $unit_name->unit = $request->unit;
        $unit_name->save();

        Session::flash('alert-success', 'Item Unit Name updated successfully');
        return redirect('/masters/item-units');

        }
    }

    /**
     * Remove the specified resource from storage.
     *
     * @param  \App\ItemUnit  $item_unit
     * @return \Illuminate\Http\Response
     */
    public function destroy($item_unit_id)
    {
        $item_unit = ItemUnit::findOrFail($item_unit_id);
        $item_unit->delete();
        Session::flash('alert-success', 'Item Unit deleted successfully');
        return redirect('/masters/item-units');
    }

    public function softdelete($item_unit_id)
    {
        $item_unit = ItemUnit::findOrFail($item_unit_id);
        $item_unit->forceDelete();
        Session::flash('alert-success', 'Item Unit deleted successfully');
        return redirect('/masters/item-units');
    }

    public function item_unit($id)
    {
        $item_name = ItemName::Find($id);
        $unit = ItemUnit::Find($item_name->unit_id);

        if($unit) {
            return response()->json(['status' => 'success', 'unit_name'  => $unit->unit, 'unit_id' => $unit->id]);
        }else{
            return response()->json(['status' => 'not-found']);
        }
    }
}
