<?php

namespace App\Http\Controllers\Transaction;
use App\Http\Controllers\Controller;
use App\PaymentVoucher;
use App\PaymentVoucherItem;
use Illuminate\Http\Request;

use Auth;
use DB;
use Storage;
use Session;
use App\Supplier;
use App\AccountMaster;
use App\AccountMastersAccountsDetail;
use App\AccountMasterTrack;
use App\ClassOfAccount;
use App\Currency;
use App\PurchaseInvoice;
use App\Services\Utils;
use App\Services\Slug;


class PaymentVoucherController extends Controller
{
    protected $utils;
    protected $slug;

    public function __construct(Utils $utils, Slug $slug)
    {
        $this->utils = $utils;
        $this->slug = $slug;
    }

    /**
     * Display a listing of the resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function index()
    {
        $user = Auth::user();
        if (!$user->can('payment-voucher')) {
            abort(403, 'Unauthorized action.');
        }

        $location_id = $this->getLocationId();
        $payment_vouchers = PaymentVoucher::withTrashed()->with('account')->where('location_id', $location_id)->orderBy('created_at','DESC')->paginate(100);
        return view('user.transactions.payment_voucher.index', compact('payment_vouchers'));
    }

    /**
     * Show the form for creating a new resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function create()
    {
        $user = Auth::user();
        if (!$user->can('payment-voucher')) {
            abort(403, 'Unauthorized action.');
        }

        $location_id = $this->getLocationId();
        $latest_voucher = PaymentVoucher::where('location_id', $location_id)->withTrashed()->latest()->first();
        if($latest_voucher){
            $invoice_no = $latest_voucher-> invoice_no + 1;
        }else{
            $invoice_no = 1;
        }

        $account_masters = AccountMasterTrack::with(['account_master' => function($q) use ($location_id) {
            $q->where('location_id', $location_id);}])->Where('user_id', auth()->user()->id)->get();
        $cash_class = ClassOfAccount::Where('name', 'Cash in Hand')->first();
        $bank_class = ClassOfAccount::Where('name', 'Bank Account')->first();
        $cash_accounts = AccountMaster::Where('location_id', $location_id)->Where('class_of_account_id', $cash_class->id)->get();
        $bank_accounts = AccountMaster::Where('location_id', $location_id)->Where('class_of_account_id', $bank_class->id)->get();
        $currency = Currency::Where('location_id', $location_id)->first();
        $tax_accounts = AccountMaster::where('class_of_account_id', 18)->get();
        return view('user.transactions.payment_voucher.create',compact('account_masters', 'invoice_no', 'currency', 'cash_accounts', 'bank_accounts', 'tax_accounts'));
    }

    /**
     * Store a newly created resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @return \Illuminate\Http\Response
     */
    public function store_old(Request $request)
    {
        // dd($request->all());
        DB::beginTransaction();

        try {
            $location_id = $this->getLocationId();
            $payment_voucher = new PaymentVoucher();
            $payment_voucher->location_id = $location_id;
            $payment_voucher->invoice_no = $request->invoice_no;
            $payment_voucher->date = $request->date;
            $payment_voucher->account_id = $request->selected_account_id;
            $payment_voucher->amount = $request->amount;
            $payment_voucher->remaining_amount = str_replace("-", "", $request->remaining_amount);
            $payment_voucher->payment_method = $request->payment_method;
            $payment_voucher->cheaque_no = $request->cheaque_no;
            $payment_voucher->cheaque_date = $request->cheaque_date;
            $payment_voucher->bank_name = $request->bank_name;
            $payment_voucher->amount_in_words = 'amount_in_words';
            $payment_voucher->narrations = $request->narrations;
            $payment_voucher->save();

            // $items_rows=explode(',',$request->items_rows);

            foreach ($request->selected_payment_type as $key => $payment_type) {
                $payment_voucher_item = new PaymentVoucherItem();
                $payment_voucher_item-> payment_voucher_id = $payment_voucher-> id;
                $payment_voucher_item-> payment_type = $request->selected_payment_type[$key];
                $remaining = str_replace("-", "", $request->invoice_remaining[$key]);
                if ($request->selected_payment_type[$key] == "billwise") {
                    $payment_voucher_item->invoice_id = $request->select_invoice_id[$key];
                    // $payment_voucher_item->paid_amount = $request->select_invoice_value[$key];
                    $payment_voucher_item->paid_amount = (float)$request->select_invoice_value[$key] - (float)$remaining;
                }
                if ($request->selected_payment_type[$key] == "on_account") {
                    $payment_voucher_item->invoice_id = "";
                    $payment_voucher_item->paid_amount = $remaining;
                }
                $payment_voucher_item-> save();

            }

            // $account = AccountMastersAccountsDetail::Where('account_master_id', $request->selected_account_id)->first();
            // $debit = (float)$account->debit;
            // $account->debit = $debit + (float)$request->amount;
            // $account->save();

            $account = new AccountMastersAccountsDetail();
            $account->account_master_id = $request->selected_account_id;
            $account->credit = 0;
            $account->debit = $request->amount;
            $account->date = $request->date;
            $account->voucher_type = "payment";
            $account->voucher_no = $request->invoice_no;
            $account->save();


            foreach ($request->select_invoice_id as $key => $invoice_id) {
                if ($request->select_invoice_id[$key] != "") {
                    $purchase_invoice = PurchaseInvoice::Find($request->select_invoice_id[$key]);

                    $remaining = str_replace("-", "", $request->invoice_remaining[$key]);
                    $paid = (float)$request->select_invoice_value[$key] - (float)$remaining;
                    $purchase_invoice->paid_amount = (float)$purchase_invoice->paid_amount + (float)$paid;
                    $purchase_invoice->remaining_amount = $remaining;
                    $purchase_invoice->save();
                }

            }


            DB::commit();
            Session::flash('alert-success','Payment Voucher created successfully');
            return redirect('/transactions/payment-vouchers');
        }catch(Exception $e) {
            DB::rollback();
            return redirect()->back()->withErrors($e);
        }
    }

    public function store(Request $request)
    {
        // dd($request->all());
        DB::beginTransaction();

        try {
            $location_id = $this->getLocationId();
            $payment_voucher = new PaymentVoucher();
            $payment_voucher->location_id = $location_id;
            $payment_voucher->invoice_no = $request->invoice_no;
            $payment_voucher->date = $request->date;
            $payment_voucher->cash_account_id = $request->cash_accounts;
            $payment_voucher->bank_account_id = $request->bank_accounts;
            // $payment_voucher->amount = $request->amount;
            // $payment_voucher->remaining_amount = str_replace("-", "", $request->remaining_amount);
            $payment_voucher->payment_method = $request->payment_method;
            $payment_voucher->cheaque_no = $request->cheaque_no;
            $payment_voucher->cheaque_date = $request->cheaque_date;
            $payment_voucher->bank_name = $request->bank_name;
            $payment_voucher->amount_in_words = $request->amount_in_words;
            $payment_voucher->gross_total = $request->gross_total;
            $payment_voucher->net_total = $request->net_total;
            $payment_voucher->vat_amount = $request->vat_amount;
            $payment_voucher->tax_account_id= $request->tax_account_id;
            $payment_voucher->narrations = $request->narrations;
            $payment_voucher->save();

            // $items_rows=explode(',',$request->items_rows);

            foreach ($request->payment_type as $key => $payment_type) {
                $payment_voucher_item = new PaymentVoucherItem();
                $payment_voucher_item-> payment_voucher_id = $payment_voucher-> id;
                $payment_voucher_item-> payment_type = $request->payment_type[$key];
                $payment_voucher_item-> amount = $request->amount[$key];
                $payment_voucher_item-> account_id = $request->account_id[$key];
                if ($request->payment_type[$key] == "billwise") {
                    $payment_voucher_item->invoice_id = $request->select_invoice[$key];
                }
                if ($request->payment_type[$key] == "on_account") {
                    $payment_voucher_item->invoice_id = "";
                }
                $payment_voucher_item-> save();

                $account = new AccountMastersAccountsDetail();
                $account->account_master_id = $request->account_id[$key];
                $account->credit = 0;
                $account->debit = $request->amount[$key];
                $account->date = $request->date;
                $account->voucher_type = "payment";
                $account->voucher_no = $request->invoice_no;
                $account->save();
            }
            
            if ($request->payment_method == "bank") {
                $account_bank = new AccountMastersAccountsDetail();
                $account_bank->account_master_id = $request->bank_accounts;
                $account_bank->credit = $request->net_total;
                $account_bank->debit = 0;
                $account_bank->date = $request->date;
                $account_bank->voucher_type = "payment";
                $account_bank->voucher_no = $request->invoice_no;
                $account_bank->save();
            }elseif ($request->payment_method == "cash") {
                $account_cash = new AccountMastersAccountsDetail();
                $account_cash->account_master_id = $request->cash_accounts;
                $account_cash->credit = $request->net_total;
                $account_cash->debit = 0;
                $account_cash->date = $request->date;
                $account_cash->voucher_type = "payment";
                $account_cash->voucher_no = $request->invoice_no;
                $account_cash->save();
            }

            //for tax account
            $tax_account = new AccountMastersAccountsDetail();
            $tax_account->account_master_id = $request->tax_account_id;
            $tax_account->credit = 0;
            $tax_account->debit = $request->vat_amount;
            $tax_account->date = $request->date;
            $tax_account->voucher_type = "payment";
            $tax_account->voucher_no = $request->invoice_no;
            $tax_account->save();


            foreach ($request->select_invoice as $key => $invoice_id) {
                if ($request->select_invoice[$key] != "") {
                    $purchase_invoice = PurchaseInvoice::Find($request->select_invoice[$key]);
                    $purchase_invoice->paid_amount = (float)$purchase_invoice->paid_amount + (float)$request->amount[$key] ;
                    $purchase_invoice->remaining_amount = (float)$purchase_invoice->remaining_amount - (float)$request->amount[$key];
                    $purchase_invoice->save();
                }

            }


            DB::commit();
            Session::flash('alert-success','Payment Voucher created successfully');
            return redirect('/transactions/payment-vouchers');
        }catch(Exception $e) {
            DB::rollback();
            return redirect()->back()->withErrors($e);
        }
    }

    /**
     * Display the specified resource.
     *
     * @param  \App\PaymentVoucher  $paymentVoucher
     * @return \Illuminate\Http\Response
     */
    public function show(PaymentVoucher $paymentVoucher)
    {
        //
    }

    /**
     * Show the form for editing the specified resource.
     *
     * @param  \App\PaymentVoucher  $paymentVoucher
     * @return \Illuminate\Http\Response
     */
    public function edit($id)
    {
        $user = Auth::user();
        if (!$user->can('edit-payment-voucher')) {
            abort(403, 'Unauthorized action.');
        }

        $location_id = $this->getLocationId();
        $payment_voucher = PaymentVoucher::with('items')->where('id', $id)->first();

        $account_masters = AccountMasterTrack::with(['account_master' => function($q) use ($location_id) {
            $q->where('location_id', $location_id);}])->Where('user_id', auth()->user()->id)->get();

        $cash_class = ClassOfAccount::Where('name', 'Cash in Hand')->first();
        $bank_class = ClassOfAccount::Where('name', 'Bank Account')->first();
        $cash_accounts = AccountMaster::Where('location_id', $location_id)->Where('class_of_account_id', $cash_class->id)->get();
        $bank_accounts = AccountMaster::Where('location_id', $location_id)->Where('class_of_account_id', $bank_class->id)->get();
        $currency = Currency::Where('location_id', $location_id)->first();
        $tax_accounts = AccountMaster::where('class_of_account_id', 18)->get();

        return view('user.transactions.payment_voucher.edit', compact('currency','cash_accounts', 'bank_accounts','payment_voucher','account_masters', 'tax_accounts'));
        // return view('user.transactions.payment_voucher.edit', compact('currency','pass_check','payment_voucher','account_masters'));
    }

    /**
     * Update the specified resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @param  \App\PaymentVoucher  $paymentVoucher
     * @return \Illuminate\Http\Response
     */
    public function update_old(Request $request, $id)
    {
        DB::beginTransaction();

        try {
            $location_id = $this->getLocationId();
            $payment_voucher = PaymentVoucher::where('id', $id)->first();
            $payment_voucher->location_id = $location_id;
            $payment_voucher->invoice_no = $request->invoice_no;
            $payment_voucher->date = $request->date;
            $payment_voucher->account_id = $request->account_id;
            $payment_voucher->total_amount = $request->total_amount;
            $payment_voucher->amount_in_words = 'amount_in_words';
            $payment_voucher->narrations = $request->narrations;
            $payment_voucher->save();

            PaymentVoucherItem::where('payment_voucher_id', $id)->forcedelete();

            foreach ($request->payment_method as $key => $payment_method) {
                $payment_voucher_item = new PaymentVoucherItem();
                $payment_voucher_item-> payment_voucher_id = $payment_voucher-> id;
                $payment_voucher_item-> payment_method = $request->payment_method[$key];
                $payment_voucher_item-> cheaque_no = $request->cheaque_no[$key];
                $payment_voucher_item-> cheaque_date = $request->cheaque_date[$key];
                $payment_voucher_item-> bank_name = $request->bank_name[$key];
                $payment_voucher_item-> amount = $request->amount[$key];
                $payment_voucher_item-> save();
            }

            DB::commit();
            Session::flash('alert-success','Payment Voucher updated successfully');
            return redirect('/transactions/payment-vouchers');
        }catch(Exception $e) {
            DB::rollback();
            return redirect()->back()->withErrors($e);
        }
    }

    public function update(Request $request, $id)
    {
        // dd($request->all());
        DB::beginTransaction();

        try {

            $location_id = $this->getLocationId();
            $payment_voucher = PaymentVoucher::where('id', $id)->first();
            $previous_cash_account = $payment_voucher->cash_account_id;
            $previous_bank_account = $payment_voucher->bank_account_id;
            $payment_voucher->location_id = $location_id;
            $payment_voucher->invoice_no = $request->invoice_no;
            $payment_voucher->date = $request->date;
            $payment_voucher->cash_account_id = $request->cash_accounts;
            $payment_voucher->bank_account_id = $request->bank_accounts;
            $payment_voucher->payment_method = $request->payment_method;
            $payment_voucher->cheaque_no = $request->cheaque_no;
            $payment_voucher->cheaque_date = $request->cheaque_date;
            $payment_voucher->bank_name = $request->bank_name;
            $payment_voucher->amount_in_words = $request->amount_in_words;
            $payment_voucher->gross_total = $request->gross_total;
            $payment_voucher->net_total = $request->net_total;
            $payment_voucher->vat_amount = $request->vat_amount;
            $payment_voucher->tax_account_id= $request->tax_account_id;
            $payment_voucher->narrations = $request->narrations;
            $payment_voucher->save();

            // PaymentVoucherItem::where('payment_voucher_id', $id)->forcedelete();
            $payment_item = PaymentVoucherItem::where('payment_voucher_id', $id)->get();
            foreach ($payment_item as $item) {

                $account_detail = AccountMastersAccountsDetail::Where('account_master_id', $item->account_id)->where('voucher_type', "payment")->where('voucher_no', $payment_voucher->invoice_no)->first();
                $account_detail->forcedelete();
                if ($previous_cash_account) {
                    $cash_account_detail = AccountMastersAccountsDetail::Where('account_master_id', $previous_cash_account)->where('voucher_type', "payment")->where('voucher_no', $payment_voucher->invoice_no)->first();
                    $cash_account_detail->forcedelete();
                }
                if ($previous_bank_account) {
                    $bank_account_detail = AccountMastersAccountsDetail::Where('account_master_id', $previous_bank_account)->where('voucher_type', "payment")->where('voucher_no', $payment_voucher->invoice_no)->first();
                    $bank_account_detail->forcedelete();
                }
                if ($item->payment_type == "billwise") {
                    $purchase_invoice = PurchaseInvoice::Where('id', $item->invoice_id)->Where('account_id', $item->account_id)->first();
                    $purchase_invoice->paid_amount = (float)$purchase_invoice->paid_amount - (float)$item->amount;
                    $purchase_invoice->remaining_amount = (float)$purchase_invoice->remaining_amount + (float)$item->amount;
                    $purchase_invoice->save();
                }
                $item->forcedelete();
            }


            foreach ($request->payment_type as $key => $payment_type) {
                $payment_voucher_item = new PaymentVoucherItem();
                $payment_voucher_item-> payment_voucher_id = $payment_voucher-> id;
                $payment_voucher_item-> payment_type = $request->payment_type[$key];
                $payment_voucher_item-> amount = $request->amount[$key];
                $payment_voucher_item-> account_id = $request->account_id[$key];
                // $remaining = str_replace("-", "", $request->invoice_remaining[$key]);
                if ($request->payment_type[$key] == "billwise") {
                    $payment_voucher_item->invoice_id = $request->select_invoice[$key];
                    // $payment_voucher_item->paid_amount = (float)$request->select_invoice_value[$key] - (float)$remaining;
                }
                if ($request->payment_type[$key] == "on_account") {
                    $payment_voucher_item->invoice_id = "";
                    // $payment_voucher_item->paid_amount = $remaining;
                }
                $payment_voucher_item-> save();

                $account = new AccountMastersAccountsDetail();
                $account->account_master_id = $request->account_id[$key];
                $account->credit = 0;
                $account->debit = $request->amount[$key];
                $account->date = $request->date;
                $account->voucher_type = "payment";
                $account->voucher_no = $request->invoice_no;
                $account->save();
            }
            
            if ($request->payment_method == "bank") {
                $account_bank = new AccountMastersAccountsDetail();
                $account_bank->account_master_id = $request->bank_accounts;
                $account_bank->credit = $request->net_total;
                $account_bank->debit = 0;
                $account_bank->date = $request->date;
                $account_bank->voucher_type = "payment";
                $account_bank->voucher_no = $request->invoice_no;
                $account_bank->save();
            }elseif ($request->payment_method == "cash") {
                $account_cash = new AccountMastersAccountsDetail();
                $account_cash->account_master_id = $request->cash_accounts;
                $account_cash->credit = $request->net_total;
                $account_cash->debit = 0;
                $account_cash->date = $request->date;
                $account_cash->voucher_type = "payment";
                $account_cash->voucher_no = $request->invoice_no;
                $account_cash->save();
            }

            //for tax account
            $tax_account = AccountMastersAccountsDetail::Where('account_master_id', $request->tax_account_id)->Where('voucher_type', "payment")->Where('voucher_no', $payment_voucher->invoice_no)->first();
            $tax_account->account_master_id = $request->tax_account_id;
            $tax_account->credit = 0;
            $tax_account->debit = $request->vat_amount;
            $tax_account->date = $request->date;
            $tax_account->voucher_type = "payment";
            $tax_account->voucher_no = $request->invoice_no;
            $tax_account->save();

            foreach ($request->select_invoice as $key => $invoice_id) {
                if ($request->select_invoice[$key] != "") {
                    $purchase_invoice = PurchaseInvoice::Find($request->select_invoice[$key]);
                    $purchase_invoice->paid_amount = (float)$purchase_invoice->paid_amount + (float)$request->amount[$key] ;
                    $purchase_invoice->remaining_amount = (float)$purchase_invoice->remaining_amount - (float)$request->amount[$key];
                    $purchase_invoice->save();
                }

            }

            DB::commit();
            Session::flash('alert-success','Payment Voucher updated successfully');
            return redirect('/transactions/payment-vouchers');
        }catch(Exception $e) {
            DB::rollback();
            return redirect()->back()->withErrors($e);
        }
    }

    /**
     * Remove the specified resource from storage.
     *
     * @param  \App\PaymentVoucher  $paymentVoucher
     * @return \Illuminate\Http\Response
     */
    public function destroy(PaymentVoucher $paymentVoucher)
    {
        //
    }

    public function softdelete($id)
    {
        $user = Auth::user();
        if (!$user->can('cancel-payment-voucher')) {
            abort(403, 'Unauthorized action.');
        }

        $payment_voucher = PaymentVoucher::Find($id);
        $previous_cash_account = $payment_voucher->cash_account_id;
        $previous_bank_account = $payment_voucher->bank_account_id;
        $previous_tax_account = $payment_voucher->tax_account_id;
        $payment_item = PaymentVoucherItem::where('payment_voucher_id', $id)->get();
        foreach ($payment_item as $item) {

            $account_detail = AccountMastersAccountsDetail::Where('account_master_id', $item->account_id)->where('voucher_type', "payment")->where('voucher_no', $payment_voucher->invoice_no)->first();
            $account_detail->delete();
            if ($previous_cash_account) {
                $cash_account_detail = AccountMastersAccountsDetail::Where('account_master_id', $previous_cash_account)->where('voucher_type', "payment")->where('voucher_no', $payment_voucher->invoice_no)->first();
                $cash_account_detail->delete();
            }
            if ($previous_bank_account) {
                $bank_account_detail = AccountMastersAccountsDetail::Where('account_master_id', $previous_bank_account)->where('voucher_type', "payment")->where('voucher_no', $payment_voucher->invoice_no)->first();
                $bank_account_detail->delete();
            }
            if ($previous_tax_account) {
                $tax_account_detail = AccountMastersAccountsDetail::Where('account_master_id', $previous_tax_account)->where('voucher_type', "payment")->where('voucher_no', $payment_voucher->invoice_no)->first();
                $tax_account_detail->delete();
            }
            if ($item->payment_type == "billwise") {
                $purchase_invoice = PurchaseInvoice::Where('id', $item->invoice_id)->Where('account_id', $item->account_id)->first();
                $purchase_invoice->paid_amount = (float)$purchase_invoice->paid_amount - (float)$item->amount;
                $purchase_invoice->remaining_amount = (float)$purchase_invoice->remaining_amount + (float)$item->amount;
                $purchase_invoice->save();
            }
            $item->delete();
        }

        $payment_voucher->delete();


        // $invoice = PurchaseInvoice::Where('account_id', $payment_voucher->account_id)->first();
        // $paid = $invoice->paid_amount;
        // $remaining = $invoice->remaining_amount;
        // $invoice->paid_amount = (float)$paid - (float)$payment_voucher->amount;
        // $invoice->remaining_amount = (float)$remaining + (float)$payment_voucher->amount;
        // $invoice->save();

        // $account = AccountMastersAccountsDetail::Where('account_master_id', $payment_voucher->account_id)->Where('voucher_type', "payment")->Where('voucher_no', $payment_voucher->invoice_no)->first();

        // $account->delete();

        // $payment_voucher->delete();

        // $payment_voucher_items = PaymentVoucherItem::Where('payment_voucher_id', $id)->get();
        // foreach ($payment_voucher_items as $item) {
        //     $item->delete();
        // }
        Session::flash('alert-success','Payment Vouchers Cancelled successfully');
        return redirect('/transactions/payment-vouchers');
    }
}
