<?php

namespace App\Http\Controllers\Master\AccountMaster;

use Illuminate\Http\Request;
use App\Http\Controllers\Controller;

use Auth;
use DB;
use Storage;
use Session;
use Carbon\Carbon;
use App\Services\Utils;
// use App\Expense;
use App\AccountMaster;
use App\AccountMastersAccountsDetail;
use App\User;
use App\Location;
use App\LocationPermission;
use App\ClassOfAccount;
use App\AccountMasterTrack;
use Illuminate\Support\Facades\Hash;

class ExpenseController extends Controller
{

    protected $utils;

    public function __construct(Utils $utils)
    {
        $this->utils = $utils;
    }

    /**
     * Display a listing of the resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function index()
    {
        $user = Auth::user();
        if (!$user->can('expenses-account-master')) {
            abort(403, 'Unauthorized action.');
        }

        $location_id = $this->getLocationId();
        $expenses = AccountMaster::Where('location_id', $location_id)->Where('account_master_type', "Expense")->orderBy('created_at','DESC')->paginate(100);
        $count = [];
        foreach ($expenses as $expense) {
            $account_detail = AccountMastersAccountsDetail::Where('account_master_id', $expense->id)->where('voucher_type', '!=' ,"opening balance")->get();
            $count[] = count($account_detail);
        }
        return view('user.masters.account_masters.expenses.index', compact('expenses', 'count'));
    }

    /**
     * Show the form for creating a new resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function create()
    {
        $user = Auth::user();
        if (!$user->can('expenses-account-master')) {
            abort(403, 'Unauthorized action.');
        }

        $location_id = $this->getLocationId();
        $country = Location::where('id', $location_id)->first();
        $class_of_accounts = ClassOfAccount::all();
        $users = LocationPermission::with('users')->where('country_id', $country->country_id)->where('location_id', $location_id)->where('user_id', '!=', $user->id)->get();
        return view('user.masters.account_masters.expenses.create', compact('class_of_accounts', 'users'));
    }

    /**
     * Store a newly created resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @return \Illuminate\Http\Response
     */
    public function store(Request $request)
    {
        $location_id = $this->getLocationId();
        $expense = AccountMaster::Where('account_name', $request->account_name)->Where('account_master_type', "Expense")->where('location_id', $location_id)->first();
        if ($expense) {
            Session::flash('alert-danger', 'This Staff Already Exists');
            return redirect()->back();
        }
        $expense = new AccountMaster();
        $expense->location_id = $location_id;
        $expense->account_master_type = "Expense";
        if ($request->transactions_password != "") {
            $expense->transactions_password = Hash::make($request->transactions_password);
        }
        $expense->opening_balance = $request->opening_balance;
        $expense->debit_credit = $request->debit_credit;
        if ($request->report_password != "") {
            $expense->report_password = Hash::make($request->report_password);
        }
        $expense->date = $request->date;
        $expense->name_of_organisation = $request->account_name;
        $expense->account_name = $request->account_name;
        $expense->class_of_account_id = $request->class_of_account_id;
        // $expense->show_user = $request->show_user;
        $expense->maintain_account = $request->maintain_account;
        $expense->description = $request->description;
        $expense->save();
        $current_expense_id = $expense->id;

        if(isset($request->show_user)){
            foreach ($request->show_user as $key => $request->show_user) {
                $account_master_track = new AccountMasterTrack();
                $account_master_track->user_id = $request->show_user;
                $account_master_track->account_master_id = $current_expense_id;
                $account_master_track->account_master_name = "Expense";
                $account_master_track->save();
            }
        }

        if(Auth::user()->hasRole('super-admin'))
        {
            $account_master_track = new AccountMasterTrack();
            $account_master_track->user_id = Auth::user()->id;
            $account_master_track->account_master_id = $current_expense_id;
            $account_master_track->account_master_name = "Expense";
            $account_master_track->save();
        }else{
            $admins = User::role('super-admin')->get();
            foreach ($admins as $key => $admin) {
                $account_master_track = new AccountMasterTrack();
                $account_master_track->user_id = $admin->id;
                $account_master_track->account_master_id = $current_expense_id;
                $account_master_track->account_master_name = "Expense";
                $account_master_track->save();
            }
            $account_master_track = new AccountMasterTrack();
            $account_master_track->user_id = Auth::user()->id;
            $account_master_track->account_master_id = $current_expense_id;
            $account_master_track->account_master_name = "Expense";
            $account_master_track->save();
        }

        if ($request->opening_balance != "") {
            $account = new AccountMastersAccountsDetail;
            $account->account_master_id = $expense->id;
            if ($request->debit_credit == "debit") {
                $account->debit = $request->opening_balance;
                $account->credit = 0;
            }elseif ($request->debit_credit == "credit") {
                $account->debit = 0;
                $account->credit = $request->opening_balance;
            }
            $account->date = $request->date;
            $account->voucher_type = "opening balance";
            $account->save();
        }


        Session::flash('alert-success', 'Expense created successfully');
        return redirect('/masters/account-masters/expenses');
    }

    /**
     * Display the specified resource.
     *
     * @param  \App\Expense  $expense
     * @return \Illuminate\Http\Response
     */
    public function show(Expense $expense)
    {
        //
    }

    /**
     * Show the form for editing the specified resource.
     *
     * @param  \App\Expense  $expense
     * @return \Illuminate\Http\Response
     */
    public function edit($expense_id)
    {
        $user = Auth::user();
        if (!$user->can('expenses-account-master')) {
            abort(403, 'Unauthorized action.');
        }

        $location_id = $this->getLocationId();
        $country = Location::where('id', $location_id)->first();
        $class_of_accounts = ClassOfAccount::all();
        $users = LocationPermission::with('users')->where('country_id', $country->country_id)->where('location_id', $location_id)->where('user_id', '!=', $user->id)->get();
        $account_master_track = AccountMasterTrack::Where('account_master_id', $expense_id)
                                                    ->Where('account_master_name', "Expense")->pluck('user_id');
        $expense = AccountMaster::Where('id', $expense_id)->Where('account_master_type', "Expense")->first();
        return view('user.masters.account_masters.expenses.edit', compact('expense', 'class_of_accounts', 'users', 'account_master_track'));
    }

    /**
     * Update the specified resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @param  \App\Expense  $expense
     * @return \Illuminate\Http\Response
     */
    public function update(Request $request, $expense_id)
    {

        $expense = AccountMaster::Where('id', $expense_id)->Where('account_master_type', "Expense")->first();
        $previous_opening_balance = (float)$expense->opening_balance;
        $current_opening_balance = (float)$request->opening_balance;
        // $expense->transactions_password = Hash::make($request->transactions_password) ? Hash::make($request->transactions_password) : $expense->transactions_password;
        if ($request->transactions_password != "") {
            $expense->transactions_password = Hash::make($request->transactions_password);
        }
        $expense->opening_balance = $request->opening_balance;
        if ($request->report_password != "") {
            $expense->report_password = Hash::make($request->report_password);
        }
        // $expense->report_password = Hash::make($request->report_password) ? Hash::make($request->report_password) : $expense->report_password;
        $expense->date = $request->date;
        $expense->name_of_organisation = $request->account_name;
        $expense->account_name = $request->account_name;
        $expense->class_of_account_id = $request->class_of_account_id;
        // $expense->show_user = $request->show_user;
        $expense->maintain_account = $request->maintain_account;
        $expense->description = $request->description;
        $expense->save();

        $account_master_track = AccountMasterTrack::Where('account_master_id', $expense_id)
                                                    ->Where('account_master_name', "Expense")->get();
        foreach ($account_master_track as $item) {
            $item->delete();
        }
        if(isset($request->show_user)){
            foreach ($request->show_user as $key => $request->show_user) {
                $account_master_track = new AccountMasterTrack();
                $account_master_track->user_id = $request->show_user;
                $account_master_track->account_master_id = $expense_id;
                $account_master_track->account_master_name = "Expense";
                $account_master_track->save();
            }
        }

        if(Auth::user()->hasRole('super-admin'))
        {
            $account_master_track = new AccountMasterTrack();
            $account_master_track->user_id = Auth::user()->id;
            $account_master_track->account_master_id = $expense_id;
            $account_master_track->account_master_name = "Expense";
            $account_master_track->save();
        }else{
            $admins = User::role('super-admin')->get();
            foreach ($admins as $key => $admin) {
                $account_master_track = new AccountMasterTrack();
                $account_master_track->user_id = $admin->id;
                $account_master_track->account_master_id = $expense_id;
                $account_master_track->account_master_name = "Expense";
                $account_master_track->save();
            }
            $account_master_track = new AccountMasterTrack();
            $account_master_track->user_id = Auth::user()->id;
            $account_master_track->account_master_id = $expense_id;
            $account_master_track->account_master_name = "Expense";
            $account_master_track->save();
        }

        // if ($request->opening_balance != "") {
        //     $account = AccountMastersAccountsDetail::Where('account_master_id', $request->account_id)->first();

        //     if ($account != "") {
        //         if ($previous_opening_balance > $current_opening_balance) {
        //             $diff = $previous_opening_balance - $current_opening_balance;
        //             $account->credit =  (float)$account->credit - (float)$diff;
        //         }
        //         elseif ($previous_opening_balance < $current_opening_balance) {
        //             $diff = $current_opening_balance - $previous_opening_balance;
        //             $account->credit = (float)$account->credit + (float)$diff;
        //         }

        //         $account->save();

        //     }else{
        //         $account = new AccountMastersAccountsDetail();
        //         $account->account_master_id = $expense->id;
        //         $account->debit = 0;
        //         $account->credit = $request->opening_balance;
        //         $account->save();
        //     }

        // }

        Session::flash('alert-success', 'Expense updated successfully');
        return redirect('/masters/account-masters/expenses');
    }

    /**
     * Remove the specified resource from storage.
     *
     * @param  \App\Expense  $expense
     * @return \Illuminate\Http\Response
     */
    public function destroy($expense_id)
    {
        $expense = AccountMaster::Where('id', $expense_id)->Where('account_master_type', "Expense")->first();
        $expense->delete();

        $account_master_track = AccountMasterTrack::Where('account_master_id', $expense_id)
                                                    ->Where('account_master_name', "Expense")->get();
        foreach ($account_master_track as $item) {
            $item->delete();
        }

        Session::flash('alert-success', 'Expense deleted successfully');
        return redirect('/masters/account-masters/expenses');
    }

    public function softdelete($expense_id)
    {
        $expense = AccountMaster::Where('id', $expense_id)->Where('account_master_type', "Expense")->first();
        $expense->forceDelete();

        $account_master_track = AccountMasterTrack::Where('account_master_id', $expense_id)
                                                    ->Where('account_master_name', "Expense")->get();
        foreach ($account_master_track as $item) {
            $item->forceDelete();
        }

        Session::flash('alert-success', 'Expense deleted successfully');
        return redirect('/masters/account-masters/expenses');
    }
}
