<?php

namespace App\Http\Controllers\Report;

use App\AccountMaster;
use App\AccountMastersAccountsDetail;
use App\ClassOfAccount;
use App\Currency;
use App\Http\Controllers\Controller;
use App\PurchaseInvoice;
use App\PurchaseReturnVoucher;
use App\SaleInvoice;
use App\SalesReturnVoucher;
use App\Services\Utils;
use Carbon\Carbon;
use Illuminate\Http\Request;
use PDF;
use Auth;

class StatementOfAccountsController extends Controller
{
    /**
     * Display a listing of the resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function index_class(Request $request)
    {
        $user = Auth::user();
        if (!$user->can('statement-of-account-report')) {
            abort(403, 'Unauthorized action.');
        }

        $location_id = Utils::getLocation();
        $from_date = $request->from_date;
        $to_date = $request->to_date;
        $trial_balance_closing_balance = [];

        $decimal = 0;

        $currency = Currency::Where('location_id', $location_id)->first();
        if ($currency) {
            $decimal = $currency->decimal_currency;
        }

        $class_of_account = ClassOfAccount::with(['account_master' => function($q) use ($location_id) {
            $q->where('location_id', $location_id);}])->where('id', $request->class_of_account)->first();
        // return $class_of_account;
            foreach ($class_of_account->account_master as $account_master){
                $debit = 0;
                $credit = 0;
                $opening_balance = 0;
                $closing_balance = 0;
                $debit = AccountMastersAccountsDetail::Where('account_master_id', $account_master->id)->where('voucher_type', '!=', 'opening balance')->Where('date', '>=' , $from_date)->Where('date', '<=' , $to_date)->sum('debit');
                $credit = AccountMastersAccountsDetail::Where('account_master_id', $account_master->id)->where('voucher_type', '!=', 'opening balance')->Where('date', '>=' , $from_date)->Where('date', '<=' , $to_date)->sum('credit');
                $first_account = AccountMastersAccountsDetail::with('account_master')->Where('account_master_id', $account_master->id)->first();
                if($first_account){
                    $with_opening_balance_debit = AccountMastersAccountsDetail::Where('account_master_id', $account_master->id)->Where('date', '>=' , $first_account->date)->Where('date', '<=' , date('Y-m-d', strtotime('-1 day', strtotime($from_date))))->sum('debit');
                    $with_opening_balance_credit = AccountMastersAccountsDetail::Where('account_master_id', $account_master->id)->Where('date', '>=' , $first_account->date)->Where('date', '<=' , date('Y-m-d', strtotime('-1 day', strtotime($from_date))))->sum('credit');
                }else{
                    $with_opening_balance_debit = 0;
                    $with_opening_balance_credit = 0;
                }
                $voucher_type_opening_balance = AccountMastersAccountsDetail::Where('account_master_id', $account_master->id)->Where('voucher_type', 'opening balance')->Where('date', '>=' , $from_date)->Where('date', '<=' , $to_date)->first();
                if($voucher_type_opening_balance){
                    if($voucher_type_opening_balance-> debit > 0){
                        $opening_balance = $voucher_type_opening_balance-> debit;
                    }else{
                        $opening_balance = $voucher_type_opening_balance->credit;
                    }
                }else {
                    if($account_master-> account_master_type == 'Supplier'){
                            $opening_balance = ($with_opening_balance_credit) - $with_opening_balance_debit;
                    }else{
                        $opening_balance = ($with_opening_balance_debit) - $with_opening_balance_credit;
                    }
                }
                if($account_master-> account_master_type == 'Supplier'){
                    $closing_balance = ($opening_balance + $credit) - $debit;
                }else{
                    $closing_balance = ($opening_balance + $debit) - $credit;
                }
                $trial_balance_closing_balance[] = [
                    "ledger" => $account_master,
                    "closing_balance" => $closing_balance,
                    "closing_balance_decimal" => number_format($closing_balance, $decimal, '.', '')
                ];
            }
        // return $trial_balance_closing_balance;

        return view('user.reports.statement_of_accounts.index_class_of_account', compact('decimal','class_of_account', 'from_date', 'to_date', 'trial_balance_closing_balance'));
    }

    public function index_ledger(Request $request)
    {
        $user = Auth::user();
        if (!$user->can('statement-of-account-report')) {
            abort(403, 'Unauthorized action.');
        }

        // dd($request->all());
        $location_id = Utils::getLocation();
        $from_date = $request->from_date;
        $to_date = $request->to_date;
        $ledger = AccountMaster::Find($request->ledger_class);
        $accounts = AccountMastersAccountsDetail::with('account_master')->Where('account_master_id', $request->ledger_class)->Where('date', '>=' , $from_date)->Where('date', '<=' , $to_date)->get();

        $decimal = 0;

        $currency = Currency::Where('location_id', $location_id)->first();
        if ($currency) {
            $decimal = $currency->decimal_currency;
        }

        $debit = AccountMastersAccountsDetail::with('account_master')->Where('account_master_id', $request->ledger_class)->where('voucher_type', '!=', 'opening balance')->Where('date', '>=' , $from_date)->Where('date', '<=' , $to_date)->sum('debit');
        $debit_decimal = number_format($debit, $decimal, '.', '');
        $credit = AccountMastersAccountsDetail::with('account_master')->Where('account_master_id', $request->ledger_class)->where('voucher_type', '!=', 'opening balance')->Where('date', '>=' , $from_date)->Where('date', '<=' , $to_date)->sum('credit');
        $credit_decimal = number_format($credit, $decimal, '.', '');

        $opening_balance = 0;
        $opening_balance_type = '';
        // $closing_balance_type = '';

        $first_account = AccountMastersAccountsDetail::with('account_master')->Where('account_master_id', $request->ledger_class)->first();
        if($first_account){
            $old_accounts_debit = AccountMastersAccountsDetail::with('account_master')->Where('account_master_id', $request->ledger_class)->Where('date', '>=' , $first_account->date)->Where('date', '<=' , date('Y-m-d', strtotime('-1 day', strtotime($from_date))))->sum('debit');
            $old_accounts_credit = AccountMastersAccountsDetail::with('account_master')->Where('account_master_id', $request->ledger_class)->Where('date', '>=' , $first_account->date)->Where('date', '<=' , date('Y-m-d', strtotime('-1 day', strtotime($from_date))))->sum('credit');
        }else{
            $old_accounts_debit = 0;
            $old_accounts_credit = 0;
        }
        $account_type = AccountMaster::where('id', $request->ledger_class)->first();
        $voucher_type_opening_balance = AccountMastersAccountsDetail::Where('account_master_id', $request->ledger_class)->Where('voucher_type', 'opening balance')->Where('date', '>=' , $from_date)->Where('date', '<=' , $to_date)->first();

        if($voucher_type_opening_balance){
            if($voucher_type_opening_balance-> debit > 0){
                $opening_balance = $voucher_type_opening_balance-> debit;
                $opening_balance_type = 'debit';
            }else{
                $opening_balance = $voucher_type_opening_balance->credit;
                $opening_balance = -($opening_balance);
                $opening_balance_type = 'credit';
            }
        }else {

            //new logic start
            $opening_balance = $old_accounts_debit - $old_accounts_credit;
            if($opening_balance >= 0){
                $opening_balance_type = 'debit';
            }else{
                $opening_balance_type = 'credit';
                // $opening_balance = -($opening_balance);
            }
            //new logic end
        }

        //new logic start
        $closing_balance = $debit - $credit;
        if($closing_balance >= 0){
            // $closing_balance_type = 'debit';
            $closing_balance = $opening_balance + $closing_balance;
        }else{
            // $closing_balance_type = 'credit';
            $closing_balance = $opening_balance + $closing_balance;
            // $closing_balance = -($closing_balance);
        }
        //new logic end

        $opening_balance_decimal = number_format($opening_balance, $decimal, '.', '');
        $closing_balance_decimal = number_format($closing_balance, $decimal, '.', '');

        // dd($opening_balance);

        return view('user.reports.statement_of_accounts.index_ledger', compact('ledger','accounts', 'from_date', 'to_date', 'credit_decimal', 'debit_decimal', 'opening_balance_decimal', 'opening_balance_type', 'closing_balance_decimal', 'closing_balance_type', 'decimal'));
    }

    public function index_ledger_billwise(Request $request)
    {
        $user = Auth::user();
        if (!$user->can('statement-of-account-report')) {
            abort(403, 'Unauthorized action.');
        }

        // dd($request->all());
        $location_id = Utils::getLocation();
        $from_date = $request->from_date;
        $to_date = $request->to_date;
        $ledger = AccountMaster::Find($request->ledger_class);

        $decimal = 0;

        $currency = Currency::Where('location_id', $location_id)->first();
        if ($currency) {
            $decimal = $currency->decimal_currency;
        }

        $purchase_invoices = PurchaseInvoice::Where('account_id', $ledger->id)->Where('invoice_no', '!=' , '0')->Where('date', '>=' , $from_date)->Where('date', '<=' , $to_date)->get();
        foreach ($purchase_invoices as $key => $purchase_invoice) {
            $purchase_invoice-> invoice_type = 'purchase';
            $date = new Carbon($purchase_invoice->date);
            $days = $purchase_invoice->lpo_days;
            if ($days == "") {
                $due_date = $date->format('Y-m-d');
            }else{
                $due_date = $date->addDays($days)->format('Y-m-d');
            }
            $purchase_invoice->due_date = $due_date;

            if ($due_date >= Carbon::now()) {
                $diff = 0;
            }else{
                $diff = Carbon::parse(Carbon::now())->diffInDays($due_date);
            }
            $purchase_invoice->diff = $diff;
        }
        $sale_invoices = SaleInvoice::Where('account_id', $ledger->id)->Where('date', '>=' , $from_date)->Where('date', '<=' , $to_date)->get();
        foreach ($sale_invoices as $key => $sale_invoice) {
            $sale_invoice-> invoice_type = 'sales';
            $date = new Carbon($sale_invoice->date);
            $days = $sale_invoice->lpo_days;
            if ($days == "") {
                $due_date = $date->format('Y-m-d');
            }else{
                $due_date = $date->addDays($days)->format('Y-m-d');
            }
            $sale_invoice->due_date = $due_date;
            if ($due_date >= Carbon::now()) {
                $diff = 0;
            }else{
                $diff = Carbon::parse(Carbon::now())->diffInDays($due_date);
            }
            $sale_invoice->diff = $diff;
        }

        $unsorted_data  = $sale_invoices->concat($purchase_invoices);
        $collection = collect($unsorted_data);
        $sorted_data = $collection->sortBy('date')->values()->all();

        return view('user.reports.statement_of_accounts.index_ledger_billwise', compact('from_date','to_date','ledger','sorted_data','decimal'));
    }

    public function index_ledger_vatwise(Request $request)
    {
        $user = Auth::user();
        if (!$user->can('statement-of-account-report')) {
            abort(403, 'Unauthorized action.');
        }

        // dd($request->all());
        $location_id = Utils::getLocation();
        $from_date = $request->from_date;
        $to_date = $request->to_date;
        $ledger = AccountMaster::Find($request->ledger_class);

        $decimal = 0;

        $currency = Currency::Where('location_id', $location_id)->first();
        if ($currency) {
            $decimal = $currency->decimal_currency;
        }

        $sale_invoices = SaleInvoice::Where('account_id', $ledger->id)->Where('location_id', $location_id)->Where('vat_check', '1')->Where('date', '>=' , $from_date)->Where('date', '<=' , $to_date)->get();
        foreach ($sale_invoices as $key => $sale_invoice) {
            $sale_invoice-> invoice_type = 'sales';
        }
        $purchase_return = PurchaseReturnVoucher::Where('account_id', $ledger->id)->Where('location_id', $location_id)->Where('vat_check', '1')->Where('date', '>=' , $from_date)->Where('date', '<=' , $to_date)->get();
        foreach ($purchase_return as $key => $_purchase_return) {
            $_purchase_return-> invoice_type = 'purchase return';
        }
        $unsorted_sale_purchase_return_data  = $sale_invoices->concat($purchase_return);
        $collection = collect($unsorted_sale_purchase_return_data);
        $sorted_sale_purchase_return_data = $collection->sortBy('date')->values()->all();
        // return $sorted_sale_purchase_return_data;

        $purchase_invoices = PurchaseInvoice::Where('account_id', $ledger->id)->Where('location_id', $location_id)->Where('invoice_no', '!=' , '0')->Where('vat_check', '1')->Where('date', '>=' , $from_date)->Where('date', '<=' , $to_date)->get();
        foreach ($purchase_invoices as $key => $purchase_invoice) {
            $purchase_invoice-> invoice_type = 'purchase';
        }
        $sale_return = SalesReturnVoucher::Where('account_id', $ledger->id)->Where('location_id', $location_id)->Where('vat_check', '1')->Where('date', '>=' , $from_date)->Where('date', '<=' , $to_date)->get();
        foreach ($sale_return as $key => $_sale_return) {
            $_sale_return-> invoice_type = 'sale return';
        }
        $unsorted_purchase_sale_return_data  = $purchase_invoices->concat($sale_return);
        $collection = collect($unsorted_purchase_sale_return_data);
        $sorted_purchase_sale_return_data = $collection->sortBy('date')->values()->all();

        return view('user.reports.statement_of_accounts.index_ledger_vatwise', compact('from_date','to_date','ledger','sorted_purchase_sale_return_data','sorted_sale_purchase_return_data','decimal'));
    }

    public function display()
    {
        $user = Auth::user();
        if (!$user->can('statement-of-account-report')) {
            abort(403, 'Unauthorized action.');
        }

        $location_id = Utils::getLocation();
        $account_masters = AccountMaster::where('location_id', $location_id)->get();
        $class_of_accounts = ClassOfAccount::all();
        return view('user.reports.statement_of_accounts.display', compact('account_masters', 'class_of_accounts'));
    }

    public function infoLedger($id)
    {
        $location_id = Utils::getLocation();
        $account = AccountMaster::where('location_id', $location_id)->Where('id', $id)->first();

        if($account) {
            return response()->json(['status' => 'success', 'data' => $account]);
        }else {
            return response()->json(['status' => 'not-found']);
        }
    }

    public function pdfViewLedger($from_date, $to_date, $id)
    {
        $user = Auth::user();
        if (!$user->can('statement-of-account-report')) {
            abort(403, 'Unauthorized action.');
        }

        $ledger = AccountMaster::Find($id);


        $location_id = Utils::getLocation();

        $accounts = AccountMastersAccountsDetail::with('account_master')->Where('account_master_id', $id)->Where('date', '>=' , $from_date)->Where('date', '<=' , $to_date)->get();

        $decimal = 0;

        $currency = Currency::Where('location_id', $location_id)->first();
        if ($currency) {
            $decimal = $currency->decimal_currency;
        }

        $debit = AccountMastersAccountsDetail::with('account_master')->Where('account_master_id', $id)->where('voucher_type', '!=', 'opening balance')->Where('date', '>=' , $from_date)->Where('date', '<=' , $to_date)->sum('debit');
        $debit_decimal = number_format($debit, $decimal, '.', '');
        $credit = AccountMastersAccountsDetail::with('account_master')->Where('account_master_id', $id)->where('voucher_type', '!=', 'opening balance')->Where('date', '>=' , $from_date)->Where('date', '<=' , $to_date)->sum('credit');
        $credit_decimal = number_format($credit, $decimal, '.', '');

        $opening_balance = 0;
        $opening_balance_type = '';

        $first_account = AccountMastersAccountsDetail::with('account_master')->Where('account_master_id', $id)->first();
        if($first_account){
            $old_accounts_debit = AccountMastersAccountsDetail::with('account_master')->Where('account_master_id', $id)->Where('date', '>=' , $first_account->date)->Where('date', '<=' , date('Y-m-d', strtotime('-1 day', strtotime($from_date))))->sum('debit');
            $old_accounts_credit = AccountMastersAccountsDetail::with('account_master')->Where('account_master_id', $id)->Where('date', '>=' , $first_account->date)->Where('date', '<=' , date('Y-m-d', strtotime('-1 day', strtotime($from_date))))->sum('credit');
        }else{
            $old_accounts_debit = 0;
            $old_accounts_credit = 0;
        }
        $account_type = AccountMaster::where('id', $ledger->id)->first();
        $voucher_type_opening_balance = AccountMastersAccountsDetail::Where('account_master_id', $id)->Where('voucher_type', 'opening balance')->Where('date', '>=' , $from_date)->Where('date', '<=' , $to_date)->first();
        
        if($voucher_type_opening_balance){
            if($voucher_type_opening_balance-> debit > 0){
                $opening_balance = $voucher_type_opening_balance-> debit;
                $opening_balance_type = 'debit';
            }else{
                $opening_balance = $voucher_type_opening_balance->credit;
                $opening_balance = -($opening_balance);
                $opening_balance_type = 'credit';
            }
        }else {
            //new logic start
            $opening_balance = $old_accounts_debit - $old_accounts_credit;
            if($opening_balance >= 0){
                $opening_balance_type = 'debit';
            }else{
                $opening_balance_type = 'credit';
                // $opening_balance = -($opening_balance);
            }
            //new logic end
        }
        //new logic start
        $closing_balance = $debit - $credit;
        if($closing_balance >= 0){
            // $closing_balance_type = 'debit';
            $closing_balance = $opening_balance + $closing_balance;
        }else{
            // $closing_balance_type = 'credit';
            $closing_balance = $opening_balance + $closing_balance;
            // $closing_balance = -($closing_balance);
        }
        //new logic end

        $opening_balance_decimal = number_format($opening_balance, $decimal, '.', '');
        $closing_balance_decimal = number_format($closing_balance, $decimal, '.', '');

        // return view('user.reports.statement_of_accounts.index_ledger', compact('ledger','accounts', 'from_date', 'to_date', 'credit', 'debit', 'opening_balance', 'opening_balance_type', 'closing_balance'));
        $pdf = PDF::loadView('user.reports.statement_of_accounts.list_ledger', compact('ledger','accounts', 'from_date', 'to_date', 'credit_decimal', 'debit_decimal', 'opening_balance_decimal', 'opening_balance_type', 'closing_balance_decimal', 'decimal'));
        return $pdf->stream();
    }

    public function pdfViewLedgerBillwise($from_date, $to_date, $id)
    {
        $user = Auth::user();
        if (!$user->can('statement-of-account-report')) {
            abort(403, 'Unauthorized action.');
        }

        $ledger = AccountMaster::Find($id);
        $purchase_invoices = PurchaseInvoice::Where('account_id', $ledger->id)->Where('invoice_no', '!=' , '0')->Where('date', '>=' , $from_date)->Where('date', '<=' , $to_date)->get();
        foreach ($purchase_invoices as $key => $purchase_invoice) {
            $purchase_invoice-> invoice_type = 'purchase';
            $date = new Carbon($purchase_invoice->date);
            $days = $purchase_invoice->lpo_days;
            if ($days == "") {
                $due_date = $date->format('Y-m-d');
            }else{
                $due_date = $date->addDays($days)->format('Y-m-d');
            }
            $purchase_invoice->due_date = $due_date;

            if ($due_date >= Carbon::now()) {
                $diff = 0;
            }else{
                $diff = Carbon::parse(Carbon::now())->diffInDays($due_date);
            }
            $purchase_invoice->diff = $diff;
        }
        $sale_invoices = SaleInvoice::Where('account_id', $ledger->id)->Where('date', '>=' , $from_date)->Where('date', '<=' , $to_date)->get();
        foreach ($sale_invoices as $key => $sale_invoice) {
            $sale_invoice-> invoice_type = 'sales';
            $date = new Carbon($sale_invoice->date);
            $days = $sale_invoice->lpo_days;
            if ($days == "") {
                $due_date = $date->format('Y-m-d');
            }else{
                $due_date = $date->addDays($days)->format('Y-m-d');
            }
            $sale_invoice->due_date = $due_date;
            if ($due_date >= Carbon::now()) {
                $diff = 0;
            }else{
                $diff = Carbon::parse(Carbon::now())->diffInDays($due_date);
            }
            $sale_invoice->diff = $diff;
        }

        $unsorted_data  = $sale_invoices->concat($purchase_invoices);
        $collection = collect($unsorted_data);
        $sorted_data = $collection->sortBy('date')->values()->all();

        // return view('user.reports.statement_of_accounts.index_ledger_billwise', compact('from_date','to_date','ledger','sorted_data'));
        $pdf = PDF::loadView('user.reports.statement_of_accounts.list_ledger_billwise', compact('from_date','to_date','ledger','sorted_data'));
        return $pdf->stream();
    }

    public function pdfViewLedgerVatwise($from_date, $to_date, $id)
    {
        $user = Auth::user();
        if (!$user->can('statement-of-account-report')) {
            abort(403, 'Unauthorized action.');
        }

        $location_id = Utils::getLocation();
        $ledger = AccountMaster::Find($id);

        $decimal = 0;

        $currency = Currency::Where('location_id', $location_id)->first();
        if ($currency) {
            $decimal = $currency->decimal_currency;
        }

        $sale_invoices = SaleInvoice::Where('account_id', $ledger->id)->Where('location_id', $location_id)->Where('vat_check', '1')->Where('date', '>=' , $from_date)->Where('date', '<=' , $to_date)->get();
        foreach ($sale_invoices as $key => $sale_invoice) {
            $sale_invoice-> invoice_type = 'sales';
        }
        $purchase_return = PurchaseReturnVoucher::Where('account_id', $ledger->id)->Where('location_id', $location_id)->Where('vat_check', '1')->Where('date', '>=' , $from_date)->Where('date', '<=' , $to_date)->get();
        foreach ($purchase_return as $key => $_purchase_return) {
            $_purchase_return-> invoice_type = 'purchase return';
        }
        $unsorted_sale_purchase_return_data  = $sale_invoices->concat($purchase_return);
        $collection = collect($unsorted_sale_purchase_return_data);
        $sorted_sale_purchase_return_data = $collection->sortBy('date')->values()->all();
        // return $sorted_sale_purchase_return_data;

        $purchase_invoices = PurchaseInvoice::Where('account_id', $ledger->id)->Where('location_id', $location_id)->Where('invoice_no', '!=' , '0')->Where('vat_check', '1')->Where('date', '>=' , $from_date)->Where('date', '<=' , $to_date)->get();
        foreach ($purchase_invoices as $key => $purchase_invoice) {
            $purchase_invoice-> invoice_type = 'purchase';
        }
        $sale_return = SalesReturnVoucher::Where('account_id', $ledger->id)->Where('location_id', $location_id)->Where('vat_check', '1')->Where('date', '>=' , $from_date)->Where('date', '<=' , $to_date)->get();
        foreach ($sale_return as $key => $_sale_return) {
            $_sale_return-> invoice_type = 'sale return';
        }
        $unsorted_purchase_sale_return_data  = $purchase_invoices->concat($sale_return);
        $collection = collect($unsorted_purchase_sale_return_data);
        $sorted_purchase_sale_return_data = $collection->sortBy('date')->values()->all();

        // return view('user.reports.statement_of_accounts.index_ledger_vatwise', compact('from_date','to_date','ledger','sorted_purchase_sale_return_data','sorted_sale_purchase_return_data'));
        $pdf = PDF::loadView('user.reports.statement_of_accounts.list_ledger_vatwise', compact('from_date','to_date','ledger','sorted_purchase_sale_return_data','sorted_sale_purchase_return_data', 'decimal'));
        return $pdf->stream();
    }

    public function pdfViewClass($from_date, $to_date, $id)
    {
        $user = Auth::user();
        if (!$user->can('statement-of-account-report')) {
            abort(403, 'Unauthorized action.');
        }

        $location_id = Utils::getLocation();
        $trial_balance_closing_balance = [];
        $decimal = 0;

        $currency = Currency::Where('location_id', $location_id)->first();
        if ($currency) {
            $decimal = $currency->decimal_currency;
        }
        $class_of_account = ClassOfAccount::with(['account_master' => function($q) use ($location_id) {
            $q->where('location_id', $location_id);}])->where('id', $id)->first();
        // return $class_of_account;
            foreach ($class_of_account->account_master as $account_master){
                $debit = 0;
                $credit = 0;
                $opening_balance = 0;
                $closing_balance = 0;
                $debit = AccountMastersAccountsDetail::Where('account_master_id', $account_master->id)->where('voucher_type', '!=', 'opening balance')->Where('date', '>=' , $from_date)->Where('date', '<=' , $to_date)->sum('debit');
                $credit = AccountMastersAccountsDetail::Where('account_master_id', $account_master->id)->where('voucher_type', '!=', 'opening balance')->Where('date', '>=' , $from_date)->Where('date', '<=' , $to_date)->sum('credit');
                $first_account = AccountMastersAccountsDetail::with('account_master')->Where('account_master_id', $account_master->id)->first();
                if($first_account){
                    $with_opening_balance_debit = AccountMastersAccountsDetail::Where('account_master_id', $account_master->id)->Where('date', '>=' , $first_account->date)->Where('date', '<=' , date('Y-m-d', strtotime('-1 day', strtotime($from_date))))->sum('debit');
                    $with_opening_balance_credit = AccountMastersAccountsDetail::Where('account_master_id', $account_master->id)->Where('date', '>=' , $first_account->date)->Where('date', '<=' , date('Y-m-d', strtotime('-1 day', strtotime($from_date))))->sum('credit');
                }else{
                    $with_opening_balance_debit = 0;
                    $with_opening_balance_credit = 0;
                }
                $voucher_type_opening_balance = AccountMastersAccountsDetail::Where('account_master_id', $account_master->id)->Where('voucher_type', 'opening balance')->Where('date', '>=' , $from_date)->Where('date', '<=' , $to_date)->first();
                if($voucher_type_opening_balance){
                    if($voucher_type_opening_balance-> debit > 0){
                        $opening_balance = $voucher_type_opening_balance-> debit;
                    }else{
                        $opening_balance = $voucher_type_opening_balance->credit;
                    }
                }else {
                    if($account_master-> account_master_type == 'Supplier'){
                            $opening_balance = ($with_opening_balance_credit) - $with_opening_balance_debit;
                    }else{
                        $opening_balance = ($with_opening_balance_debit) - $with_opening_balance_credit;
                    }
                }
                if($account_master-> account_master_type == 'Supplier'){
                    $closing_balance = ($opening_balance + $credit) - $debit;
                }else{
                    $closing_balance = ($opening_balance + $debit) - $credit;
                }
                $trial_balance_closing_balance[] = [
                    "ledger" => $account_master,
                    "closing_balance" => $closing_balance,
                    "closing_balance_decimal" => number_format($closing_balance, $decimal, '.', '')
                ];
            }
        // return $trial_balance_closing_balance;

        // return view('user.reports.statement_of_accounts.index_class_of_account', compact('class_of_account', 'from_date', 'to_date', 'trial_balance_closing_balance'));
        $pdf = PDF::loadView('user.reports.statement_of_accounts.list_class', compact('class_of_account', 'from_date', 'to_date', 'trial_balance_closing_balance'));
        return $pdf->stream();
    }

    /**
     * Show the form for creating a new resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function create()
    {
        //
    }

    /**
     * Store a newly created resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @return \Illuminate\Http\Response
     */
    public function store(Request $request)
    {
        //
    }

    /**
     * Display the specified resource.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function show($id)
    {
        //
    }

    /**
     * Show the form for editing the specified resource.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function edit($id)
    {
        //
    }

    /**
     * Update the specified resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function update(Request $request, $id)
    {
        //
    }

    /**
     * Remove the specified resource from storage.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function destroy($id)
    {
        //
    }
}
